/******************************************************************************/
/*                  S A S   S A M P L E   L I B R A R Y                       */
/*                                                                            */
/*    NAME: alert_age_report.sas                                              */
/*   TITLE: Alert Age Report                                                  */
/* PRODUCT: SAS Anti-Money Laundering                                         */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/******************************************************************************/
/* Description: Produces a tabular report of the age of active alert,closed
/*              and suppressed alerts.  For active alerts it shows the avg
/*              and max age of all active alerts.
/*              For closed and suppressed alerts it show the avg and maximum
/*              time to close or suppress the alert.
/*
/* PARAMETERS:
/* <!-- ODS Parms -->
/*      outputDirectory = DIRECTORY FOR THIS REPORT
/*      outputBody = REPORT HTML BODY NAME:  body.htm
/*      outputDevice = GRAPHICS RENDERING:  javaimg/actximg
/*      _outputStyle = STYLE TO APPLY:  sasweb, banker etc
/*      _outputStyle._prt  = STYLE TO APPLY To PROC PRINT
/* <!-- Report Parms -->
/*      title = REPORT TITLE
/*        chart_report_YN  = N   NO bar chart is the default
/*      chart_type = VBAR3D;
/*      bar_shape = SHAPE OF BARS:  block
/*      color1 = ;  BAR COLORS
/*      color2 =  ;   BAR COLORS
/* <!-- Called routines and macros -->
/*      %aml_rcset
/*      %put_aml_error
/*      %aml_execute_command
/*      %no_record_htm
/*      %label_maker
/******************************************************************************/

/* ************************************************************************ */
/*  Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA       */
/*  ---All Rights Reserved.                                                 */
/* ************************************************************************ */

%macro alert_age_report / DES='AML51HF1';

    /*--------------------------------------------------------
     * Make sure the AML start datetime has been set.
     * Assign its value to macro variable aml_start_datetime.
     *--------------------------------------------------------*/
    %aml_start_datetime_read;
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %goto MACRO_END2;
    %end;

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    /*+  macro  aml_get_active_alerts                             */
    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    %macro aml_get_active_alerts;

        %global actual_number_&ALERT_TYPE   ;

        /* only do if there are active alerts */
        %if &num_act ne 0 %then  %do;

            /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
            /*  GET ACTIVE ALERT RECORDS                                      */
            /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
            PROC SQL NOPRINT;
            CREATE TABLE WORK.&ALERT_TYPE._ALERTS AS
            SELECT
                FSK_ALERT.ALERT_ID                     ,
                FSK_ALERT.PRODUCT_TYPE                 ,
                FSK_ALERT.ALERT_STATUS_CODE            FORMAT=$status.,
                FSK_ALERT.MONEY_LAUNDERING_RISK_SCORE  ,
                FSK_ALERT.TERROR_FINANCING_RISK_SCORE  ,
                FSK_ALERT.ALERT_DESCRIPTION            ,
                FSK_ALERT.PRIMARY_ENTITY_LEVEL_CODE    FORMAT=$subject.,
                FSK_ALERT.PRIMARY_ENTITY_NUMBER        ,
                FSK_ALERT.PRIMARY_ENTITY_NAME          ,
                FSK_ALERT.SCENARIO_NAME                ,
                FSK_ALERT.ACTUAL_VALUES_TEXT           ,
                FSK_ALERT.CREATE_DATE                  FORMAT=nldatm.,
                FSK_ALERT.CREATE_USER_ID
            FROM SEG_KC.FSK_ALERT AS FSK_ALERT
            WHERE FSK_ALERT.ALERT_STATUS_CODE IN ("&alert_type")
            ORDER BY FSK_ALERT.CREATE_DATE DESCENDING;
            QUIT;

            %aml_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to create table: &alert_type._ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            %let actual_number_&alert_type =  &sqlobs   ;

            DATA  WORK.&ALERT_TYPE._ALERTS  ;
                SET WORK.&ALERT_TYPE._ALERTS  END=EOF ;
                drop count todays_date;
                LABEL ALERT_AGE = "&active_days";

                /* -----------------------------------------------------------*/
                /* Generate LABELS                                            */
                /* -----------------------------------------------------------*/
                %label_maker(basefile=alert_age_meta
                            ,table_name=ALERT_AGE_REPORT_LABELS_META
                            ,generate=label ) ;

                if _N_=1 THEN DO ;
                    retain todays_date count;
                    todays_date = datepart(&aml_start_datetime);
                    count = 0;
                END;

                ALERT_AGE = todays_date - datepart(create_date);
                count+1;

                output work.&alert_type._ALERTS  ;

                IF EOF THEN DO;
                    call symput('actual_number_act',trim(left(put(count,8.))));
                END;
            RUN;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to update table: &alert_type._ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            %let actual_number_&alert_type =  &sqlobs   ;
        %end;
        %else %do;
            %let actual_number_&alert_type = 0;
        %end;

        %if &&&actual_number_&alert_type ne 0 %then  %do;   /* we have alerts */

            /* SORT ASCENDING */
            proc sort data=work.&alert_type._alerts
                out=work.avg_&alert_type._alerts_sorted;
                by alert_age;
            run   ;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to create table: AVG_&ALERT_TYPE._ALERTS_SORTED;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
            data work.avg_&alert_type._alerts;
                set work.avg_&alert_type._alerts_sorted;
                if _n_ gt &_display_limit then delete;
            run;


            /* SORT DESCENDING */
            proc sort data=work.&alert_type._alerts
                out=work.max_&alert_type._alerts_sorted;
                    by descending alert_age;
            run   ;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to create table: MAX_&ALERT_TYPE._ALERTS_SORTED;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
            data work.max_&alert_type._alerts;
                set work.max_&alert_type._alerts_sorted;
                if _n_ gt &_display_limit then delete;
            run;


            /* SUMMARIZE ALERTS - SUM NUMBER - AVERAGE, MAXIMUM AGE */
            proc means data=work.&alert_type._alerts  maxdec=1 noprint ;
                var alert_age;
                class alert_status_code ;
                output  out=work.&alert_type._ALERTS_SUMMARY
                    n(ALERT_AGE) =   NUM_ALERT_AGE
                    sum(ALERT_AGE) = SUM_ALERT_AGE
                    mean(ALERT_AGE) = AVG_ALERT_AGE
                    max(ALERT_AGE) = MAX_ALERT_AGE  ;
                ways 1;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to create table: &ALERT_TYPE._ALERTS_SUMMARY;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            data _null_    ;
                set  work.&alert_type._ALERTS_SUMMARY ;
                %global   avg_age_&alert_type  max_age_&alert_type ;
                call symput("avg_age_&alert_type",trim(left(put(AVG_ALERT_AGE,8.1))));
                call symput("max_age_&alert_type",trim(left(put(max_ALERT_AGE,8.))));
            run;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to read table: &ALERT_TYPE._ALERTS_SUMMARY;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
        %end;
        %else %do;     /* we have no alerts available or selected */
                       /* Create Dummy files for alerts */
            PROC SQL NOPRINT;
                CREATE TABLE work.&alert_type._ALERTS
                ( ALERT_STATUS_CODE   char(3) ,
                Alert_age int )   ;
            quit;
            %aml_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to SQL create table: &alert_type._ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            PROC SQL NOPRINT;
                insert into work.&alert_type._ALERTS
                set ALERT_STATUS_CODE ="&alert_type",
                Alert_age=0;
            quit;
            %aml_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to SQL update table: &alert_type._ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            PROC SQL NOPRINT;
                CREATE TABLE work.&alert_type._ALERTS_SUMMARY
                    (ALERT_STATUS_CODE   char(3) ,
                    NUM_ALERT_AGE   int,
                    SUM_ALERT_AGE   int,
                    AVG_ALERT_AGE   int,
                    MAX_ALERT_AGE   int);
            quit;
            %aml_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to SQL create table: &alert_type._ALERTS_SUMMARY;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            PROC SQL NOPRINT;
                 insert into work.&alert_type._ALERTS_SUMMARY
                 set ALERT_STATUS_CODE ="&alert_type",
                 NUM_ALERT_AGE=0,
                 SUM_ALERT_AGE=0,
                 AVG_ALERT_AGE=0,
                 MAX_ALERT_AGE=0 ;
            quit;
            %aml_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to SQL update table: &alert_type._ALERTS_SUMMARY;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

        %end;

    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    %mend aml_get_active_alerts;


    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* macro  aml_get_alert_events                                 */
    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    %macro aml_get_alert_events;
        %global actual_number_&alert_type   ;

        %if  num_&alert_type ne 0 %then  %do;

            %if %upcase(&dbkcflavor) ne MYSQL %then %do;
                proc sql;
                    create table work.most_recent_events as
                    select a.alert_id, max(a.create_date) as max_create_date FORMAT=NLDATM30.
                    from SEG_KC.fsk_alert_event as a
                    where a.event_type_code in ("&event_code_value")
                    group by alert_id;
                quit;
            %end;
            %else %do;
                %let tic=%str(%');
                proc sql;
                  connect to &dbkcflavor as &dbkcflavor (user=&segKCDBUser
                                    password=&segKCDBUserPwd &segKCDBConnOpts);
                    create table work.most_recent_events as
                    select alert_id, max_create_date format=nldatm.
                    from connection to &dbkcflavor
                    (select a.alert_id, cast(max(a.create_date)as datetime) as max_create_date
                    from &segkcschema..fsk_alert_event as a
                    where a.event_type_code=&tic&event_code_value&tic
                    group by alert_id  );
                  disconnect from &dbkcflavor;
                quit;
            %end;


            PROC SQL  NOPRINT;
                CREATE TABLE work.&alert_type._alerts AS
                SELECT
                    FSK_ALERT.ALERT_ID                     ,
                    FSK_ALERT.PRODUCT_TYPE                 ,
                    FSK_ALERT.ALERT_STATUS_CODE            FORMAT=$status.,
                    FSK_ALERT.MONEY_LAUNDERING_RISK_SCORE  ,
                    FSK_ALERT.TERROR_FINANCING_RISK_SCORE  ,
                    FSK_ALERT.ALERT_DESCRIPTION            ,
                    FSK_ALERT.PRIMARY_ENTITY_LEVEL_CODE    FORMAT=$subject.,
                    FSK_ALERT.PRIMARY_ENTITY_NUMBER        ,
                    FSK_ALERT.PRIMARY_ENTITY_NAME          ,
                    FSK_ALERT.SCENARIO_NAME                ,
                    FSK_ALERT.ACTUAL_VALUES_TEXT           ,
                    FSK_ALERT.CREATE_DATE                  FORMAT=nldatm.,
                    FSK_ALERT.CREATE_USER_ID               ,
                    DATEPART(E.max_create_date) - DATEPART(FSK_ALERT.CREATE_DATE) FORMAT=8. AS  ALERT_AGE
                FROM SEG_KC.FSK_ALERT AS FSK_ALERT
                INNER JOIN
                WORK.MOST_RECENT_EVENTS AS E ON (FSK_ALERT.ALERT_ID = E.ALERT_ID)
                WHERE FSK_ALERT.ALERT_STATUS_CODE IN ("&alert_type");
            QUIT;

            %aml_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to create table: &alert_type._ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            %let actual_number_&alert_type =  &sqlobs   ;
        %end;
        %else %do;
            %let actual_number_&alert_type = 0;
        %end;

        %if &&&actual_number_&alert_type ne 0 %then  %do;   /* we have alerts */

            data work.&alert_type._ALERTS;
                set work.&alert_type._ALERTS;
                /* -----------------------------------------------------------*/
                /* Generate LABELS                                            */
                /* -----------------------------------------------------------*/
                %label_maker(basefile=alert_age_meta
                            ,table_name=ALERT_AGE_REPORT_LABELS_META
                            ,generate=label ) ;

            %if &alert_type = SUP %then %do;
                LABEL ALERT_AGE = "&suppress_days";
            %end;
            %else %if &alert_type = CLS %then %do;
                LABEL ALERT_AGE = "&close_days";
            %end;
            %else %do;
                LABEL ALERT_AGE = "&active_days";
            %end;

            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to update table: &ALERT_TYPE._ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            /* SORT ASCENDING */
            proc sort data=work.&alert_type._alerts
                out=work.avg_&alert_type._alerts_sorted;
                    by alert_age;
            run   ;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to create table: AVG_&ALERT_TYPE._ALERTS_SORTED;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            data work.avg_&alert_type._alerts;
                set work.avg_&alert_type._alerts_sorted;
                if _n_ gt &_display_limit then delete;
            run;

            /* SORT DESCENDING */
            proc sort data=work.&alert_type._alerts
                out=work.max_&alert_type._alerts_sorted;
                    by descending alert_age;
            run   ;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to create table: MAX_&ALERT_TYPE._ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            data work.max_&alert_type._alerts;
                set work.max_&alert_type._alerts_sorted;
                if _n_ gt &_display_limit then delete;
            run;

            /* SUMMARIZE ALERTS - SUM NUMBER - AVERAGE, MAXIMUM AGE */
            proc means data=work.&alert_type._alerts  maxdec=1 noprint ;
                var alert_age;
                class alert_status_code ;
                output  out=work.&alert_type._ALERTS_SUMMARY
                    n(ALERT_AGE) =   NUM_ALERT_AGE
                    sum(ALERT_AGE) = SUM_ALERT_AGE
                    mean(ALERT_AGE) = AVG_ALERT_AGE
                    max(ALERT_AGE) = MAX_ALERT_AGE  ;
                ways 1;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to create table: &ALERT_TYPE._ALERTS_SUMMARY;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            data _null_    ;
                set  work.&alert_type._ALERTS_SUMMARY ;
                %global   avg_age_&alert_type  max_age_&alert_type ;
                call symput("avg_age_&alert_type",trim(left(put(AVG_ALERT_AGE,8.1))));
                call symput("max_age_&alert_type",trim(left(put(max_ALERT_AGE,8.))));
            run;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to read table: &ALERT_TYPE._ALERTS_SUMMARY;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

        %end;
        %else %do;     /* we have no alerts available or selected */
                       /* Create Dummy files for alerts */
            PROC SQL NOPRINT;
                 CREATE TABLE work.&alert_type._ALERTS
                 ( ALERT_STATUS_CODE   char(3) ,
                 Alert_age int )   ;
            quit;
            %aml_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to SQL create table: &alert_type._ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            PROC SQL NOPRINT;
                insert into work.&alert_type._ALERTS
                set ALERT_STATUS_CODE ="&alert_type",
                Alert_age=0;
            quit;
            %aml_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to SQL update table: &alert_type._ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            PROC SQL NOPRINT;
                CREATE TABLE work.&alert_type._ALERTS_SUMMARY
                    (ALERT_STATUS_CODE   char(3) ,
                    NUM_ALERT_AGE   int,
                    SUM_ALERT_AGE   int,
                    AVG_ALERT_AGE   int,
                    MAX_ALERT_AGE   int);
            quit;
            %aml_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to SQL create table: &alert_type._ALERTS_SUMMARY;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;

            PROC SQL NOPRINT;
                insert into work.&alert_type._ALERTS_SUMMARY
                set ALERT_STATUS_CODE ="&alert_type",
                NUM_ALERT_AGE=0,
                SUM_ALERT_AGE=0,
                AVG_ALERT_AGE=0,
                MAX_ALERT_AGE=0 ;
            quit;
            %aml_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to SQL update table: &alert_type._ALERTS_SUMMARY;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
        %end;
    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    %mend aml_get_alert_events;


    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* macro  aml_generate_summary_file                            */
    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    %macro aml_generate_summary_file;

        /* ---------------------------------------------------------- */
        /* Create table of alert codes                                */
        /* ---------------------------------------------------------- */

        DATA WORK.VALID_STATUS_COUNT;
            set work.ACT_ALERTS_SUMMARY
                work.CLS_ALERTS_SUMMARY
                work.SUP_ALERTS_SUMMARY ;

            label alert_status_code = 'Alert Status'   ;
                  alert_status  = 'Alert Status'   ;
            drop _type_ _freq_;
            format alert_status_code $status.  alert_status  $3.;
            alert_status = alert_status_code;
        run;
        %aml_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _amlmsg_= NOTE: AML: Unable to create table: VALID_STATUS_COUNT;
            %put_aml_error;
            %let error_sw =1;
            %return ;
        %end;

        /* ---------------------------------------------------------- */
        /* Create macro variables of lists of codes that have records */
        /* ---------------------------------------------------------- */
        proc sql noprint;
            select alert_status ,
            num_alert_age ,
            count(alert_status)
            into
                :status_list separated by ',' ,
                :count_list separated by ',' ,
                :count_of_status_selected
            from work.valid_status_count;
        quit;
        %aml_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _amlmsg_= NOTE: AML: Unable to SQL read table: VALID_STATUS_COUNT;
            %put_aml_error;
            %let error_sw =1;
            %return ;
        %end;

        /* ------------------------------------------------------------- */
        /* Allocate catalog for source                                   */
        /* ------------------------------------------------------------- */
        proc datasets lib=work nolist mt=cat;
            delete codefile;
        quit;
        filename tempcode catalog "work.codefile.tempcode.source";

        /* ------------------------------------------------------- */
        /* This macro generates the code necessary to create a
        /* summary file by status code grouped by Average and
        /* maximum values along with associated HTML links
        /*
        /* EXTERNAL INPUT FILES AND VARIABLES:
        /* VARIABLES:
        /* &status_list: list of status codes with alerts
        /* &count_of_status_selected: number of status codes with
        /*      alerts
        /*
        /*
        /* ------------------------------------------------------- */
        data _null_;
            format line $512.;
            file tempcode lrecl=256;

            put 'data work.combined_summary_table;' ;
            put 'set ';

            do i = 1 to &count_of_status_selected;
                line = 'work.' || trim(left(scan("&status_list",i))) || '_ALERTS_SUMMARY';
                put   @6 line ;
            end;

            put '   ;';
            put '*---------------------------------------------------------; ';
            put '* Housekeeping                                            ; ';
            put '*---------------------------------------------------------; ';
            put 'drop  avg_alert_age max_alert_age sum_alert_age  ;';
            put 'format Age $8. Days 8. listing_link $50.  Agetab $100. ; ';
            put 'format alert_status_code $rptstat.;  ';

            put 'label alert_status_code = "Alert Status"  ';
            put '      Age = "Age"  ';
            put '      Agetab ="Age" ; ';

            put '*---------------------------------------------------------; ';
            put '* Create record for average value                         ; ';
            put '*---------------------------------------------------------; ';

            put ' Age = "Avg. Age";  ';
            put ' Days = avg_alert_age; ';
            do i = 1 to &count_of_status_selected;
                line = 'if lowcase(alert_status_code) = "' || lowcase(trim(left(scan("&status_list",i))))  || '" then do; ';
                put line;

                if trim(left(scan("&count_list",i))) ne '0' then do;
                    line = '   listing_link =  ''HREF="avg_age_list_' || lowcase(strip(scan("&status_list",i)) ) || '.htm"'';' ;
                    put @4 line;
                    line = '   Agetab =''<a  HREF="avg_age_list_' || lowcase(strip(scan("&status_list",i))) ||  '.htm">' || "&average" || '</a>'';   ';
                    put @4 line;
                end;
                else do;
                    line = '   listing_link =  ''HREF="no_rec.htm"''   ;' ;
                    put @4 line;
                    line = '   Agetab =''<a  HREF="no_rec.htm"">' || "&average" || '</a>'' ;   ';
                    put @4 line;
                end;

                put 'end; ';
                if i  lt   &count_of_status_selected then
                    put 'else;   ';
            end;
            put    'output work.combined_summary_table;  ';

            put '*---------------------------------------------------------; ';
            put '* Create record for maximum value                         ; ';
            put '*---------------------------------------------------------; ';

            put ' Age = "Max. Age";  ';
            put ' Days = max_alert_age; ';

            do i = 1 to &count_of_status_selected;
                line = 'if lowcase(alert_status_code) = "' || lowcase(trim(left(scan("&status_list",i))))  || '" then do; ';
                put line;


                if trim(left(scan("&count_list",i))) ne '0' then do;
                    line = '   listing_link =  ''HREF="max_age_list_' || lowcase(strip(scan("&status_list",i)))  || '.htm"'';' ;
                    put @4 line;
                    format max_part1 max_part2 max_part3
                    max_part4 max_part5 max_part6
                    max_part7 max_part8 $40.;


                    max_part1 = 'Agetab = ';
                    max_part2 = "'<a  HREF=" ;
                    max_part3 = '"max_age_list_';
                    max_part4 = lowcase(strip(scan("&status_list",i)));
                    max_part5 = '.htm">';
                    max_part6 = "&maximum" ;
                    max_part7 = '</a>';
                    max_part8 = "';";

                    line = '    ' || catt(max_part1,max_part2,max_part3,max_part4,max_part5,max_part6,
                    max_part7,max_part8);
                    put @4 line;

                end;
                else do;
                    line = '   listing_link =  ''HREF="no_rec.htm"''   ;' ;
                    put @4 line;
                    line = '   Agetab =''<a  HREF="no_rec.htm"">' || "&maximum" || '</a>'' ;   ';
                    put @4 line;
                end;

                put 'end; ';
                if i  lt   &count_of_status_selected then
                    put 'else;   ';
            end;
            put    'output work.combined_summary_table;  ';
            put    'run;  ';
        run;
        %aml_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _amlmsg_= NOTE: AML: Unable to generate source file: TEMPCODE;
            %put_aml_error;
            %let error_sw =1;
            %return ;
        %end;

    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    %mend aml_generate_summary_file;


    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* macro  aml_create_age_charts                                */
    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    %macro aml_create_age_charts ;

        /* ---------------------------------------------------------*/
        /* Setup the output environment
        /* ---------------------------------------------------------*/
        /* Delete output directory if it exists
        /* use lower case 'deldir' to indicate that you want to
        /* remove directory - macro converts to proper code
        /* depending on platform  - unix or windows                 */

        /* If output directory exists --- remove old reports        */
        %if %sysfunc(fileexist(&outputDirectory)) %then %do;
            %aml_execute_command(deldir  &outputDirectory);
        %end;

        /* Create output directory                                  */
        %aml_execute_command(mkdir &outputDirectory);

        /*  +++++++++++++++++++++++++++++++++++++++++++++++++ */

        OPTIONS   DEV=&_outputdevice ;/* Chart Area Options */
        GOPTIONS  xpixels=600  ypixels=400;
        ODS LISTING CLOSE;
        ODS HTML PATH="&outputDirectory"     (URL=NONE)
            BODY="&outputBody"
            NOGTITLE NOGFOOTNOTE
            STYLE=&_outputstyle   ;

        /*  +++++++++++++++++++++++++++++++++++++++++++++++++ */
        /*  GENERATE CHART --- YES or NO                      */
        /*  +++++++++++++++++++++++++++++++++++++++++++++++++ */
        %if %upcase(&_chart_report_YN) = Y %then %do;
            /*  +++++++++++++++++++++++++++++++++++++++++++++++++ */
            /*  CHART TYPE --- BLOCK or BAR                       */
            /*  +++++++++++++++++++++++++++++++++++++++++++++++++ */
            %if &_chart_type ne BLOCK %then %do;
                /* BAR CHART --- Either VBAR or HBAR */
                AXIS1 label=('Days') MINOR=NONE;
                AXIS2 label=('') MINOR=NONE  value=(height=.8);
                AXIS3 LABEL=('') ;
                &_color1;
                &_color2;

                PROC GCHART DATA=work.combined_summary_table;
                    &_chart_type    Age /
                    GROUP=ALERT_STATUS_CODE
                    SUMVAR=Days
                    RAXIS=AXIS1
                    MAXIS=AXIS2          noframe
                    GAXIS=AXIS3
                    outside=sum
                    HTML=listing_link
                    AUTOREF
                    SHAPE=&_bar_shape
                    PATTERNID=MIDPOINT
                    TYPE=sum
                    COUTLINE=BLACK
                ;
            %end;
            /* +++++++++++++++++++++++++++++++++++++++++++++++++ */
            %else %do;
                /* BLOCK CHART */
                AXIS1 LABEL=( JUSTIFY=LEFT 'Days' ) MINOR=NONE  ;
                AXIS2 LABEL=( JUSTIFY=CENTER 'Alert Status' ) MINOR=NONE ;
                AXIS3 LABEL=( JUSTIFY=LEFT ' ' ) MINOR=NONE ;
                &_color1;
                &_color2;

                PROC GCHART DATA=work.combined_summary_table;
                    BLOCK ALERT_STATUS_CODE /
                    GROUP=Age
                    SUMVAR=Days
                    RAXIS=AXIS1
                    MAXIS=AXIS2
                    GAXIS=AXIS3
                    DISCRETE
                    HTML=listing_link
                    SHAPE=block
                    PATTERNID=GROUP
                    TYPE=sum
                ;
            %end;
                title "&_title";
            /*  FOOTNOTE8 height=&_fheight justify=left "&aml_footnote_date" justify=right "&aml_footnote_version";*/
            run;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to generate charts: PROC GCHART;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
        %end;
        TITLE; FOOTNOTE;

        /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
        /* -------------------------------------------------------- */
        /* Output Tabulate Table of Values                          */
        /* -------------------------------------------------------- */

        /*  Build special class variable with the number of alerts in class appended to it*/
        DATA tabulate_report;
            SET work.combined_summary_table;
            format alert_status $80. ;
            alert_status = trim(left(put(alert_status_code,$rptstat.))) ||   ' ' ||
            trim(left(put(alert_status_code,$rpttab.))) || ' ' ||  trim(left(put(num_alert_age,8.))) || ')' ;
        run;
        title "&_title";
        /*  FOOTNOTE8 height=&_fheight justify=left "&aml_footnote_date" justify=right "&aml_footnote_version";*/

        PROC TABULATE DATA=work.tabulate_report
            STYLE={JUST=CENTER VJUST=BOTTOM} FORMAT=COMMA8.;
            VAR Days ;
            CLASS ALERT_STATUS / MISSING ORDER=UNFORMATTED;
            CLASS Agetab / MISSING ORDER=UNFORMATTED;
            TABLE /* Row Dimension */
            ALERT_STATUS={LABEL=' '}*
            Agetab={LABEL=' '},
            /* Column Dimension */
            Days={LABEL=' ' STYLE={NOBREAKSPACE=ON}}*
            Sum={LABEL=' ' STYLE={NOBREAKSPACE=ON CELLWIDTH=10in} } ;
        RUN;
        %aml_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _amlmsg_= NOTE: AML: Unable to generate output table: PROC TABULATE;
            %put_aml_error;
            %let error_sw =1;
            %return ;
        %end;

        TITLE;FOOTNOTE;
        ODS _ALL_ CLOSE;
        ODS LISTING;
    %mend aml_create_age_charts ;


    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* MACRO  aml_create_age_reports                               */
    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    %macro aml_create_age_reports;

        /* ------------------------------------------------------------------ */
        /* AVG_ACT_ALERTS                                                     */
        /* ------------------------------------------------------------------ */

        %if &actual_number_act ne 0 %then  %do;
            OPTIONS   DEV=&_outputdevice ;
            ODS LISTING CLOSE;
            ODS HTML PATH="&outputDirectory"       (URL=NONE)
                BODY='avg_age_list_act.htm'
                NOGTITLE NOGFOOTNOTE
                STYLE=&_outputStyle._prt   ;

            TITLE1  "&title1_active &title1_ascending";
            TITLE2  "&limit_act &title2_of &num_act &title2_alerts";
            PROC PRINT DATA=WORK.AVG_ACT_ALERTS LABEL Split='_'  NOOBS;
                VAR &Varlist1 &Varlist2 &Varlist3 &Varlist4 ;
            RUN;
            ODS _ALL_ CLOSE;
            ODS LISTING;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to generate output detail report: AVG_ACT_ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
        %end;

        /* ------------------------------------------------------------------ */
        /* AVG_CLS_ALERTS                                                     */
        /* ------------------------------------------------------------------ */
        %if  &actual_number_cls ne 0 %then  %do;
            OPTIONS   DEV=&_outputdevice ;
            ODS LISTING CLOSE;
            ODS HTML PATH="&outputDirectory"       (URL=NONE)
                BODY='avg_age_list_cls.htm'
                NOGTITLE NOGFOOTNOTE
                STYLE=&_outputStyle._prt   ;

            TITLE1  "&title1_closed &title1_ascending";
            TITLE2  "&limit_cls &title2_of &num_cls &title2_alerts";
            PROC PRINT DATA=WORK.AVG_CLS_ALERTS LABEL Split='_' NOOBS;
                VAR &Varlist1 &Varlist2 &Varlist3 &Varlist4 ;
            RUN;
            ODS _ALL_ CLOSE;
            ODS LISTING;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to generate output detail report: AVG_CLS_ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
        %end;

        /* ------------------------------------------------------------------ */
        /* AVG_SUP_ALERTS                                                     */
        /* ------------------------------------------------------------------ */
        %if  &actual_number_sup ne 0 %then  %do;
            OPTIONS   DEV=&_outputdevice ;
            ODS LISTING CLOSE;
            ODS HTML PATH="&outputDirectory"       (URL=NONE)
                BODY='avg_age_list_sup.htm'
                NOGTITLE NOGFOOTNOTE
                STYLE=&_outputStyle._prt   ;

            TITLE1  "&title1_suppressed &title1_ascending";
            TITLE2  "&limit_sup &title2_of &num_sup &title2_alerts";
            proc print data=work.avg_sup_alerts label Split='_'  NOOBS;
                VAR &Varlist1 &Varlist2 &Varlist3 &Varlist4 ;
            RUN;
            ODS _ALL_ CLOSE;
            ODS LISTING;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to generate output detail report: AVG_SUP_ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
        %end;

        /* ------------------------------------------------------------------ */
        /* MAX_ACT_ALERTS                                                     */
        /* ------------------------------------------------------------------ */
        %if  &actual_number_act ne 0 %then  %do;
            OPTIONS   DEV=&_outputdevice ;
            ODS LISTING CLOSE;
                ODS HTML PATH="&outputDirectory"       (URL=NONE)
                BODY='max_age_list_act.htm'
                NOGTITLE NOGFOOTNOTE
                STYLE=&_outputStyle._prt   ;
            TITLE1  "&title1_active &title1_descending";
            TITLE2  "&limit_act &title2_of &num_act &title2_alerts";
            PROC PRINT DATA=WORK.MAX_ACT_ALERTS LABEL Split='_' NOOBS;
                VAR &Varlist1 &Varlist2 &Varlist3 &Varlist4 ;
            RUN;
            ODS _ALL_ CLOSE;
            ODS LISTING;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to generate output detail report: MAX_ACT_ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
        %end;

        /* ------------------------------------------------------------------ */
        /* MAX_CLS_ALERTS                                                     */
        /* ------------------------------------------------------------------ */
        %if  &actual_number_cls ne 0 %then  %do;
            OPTIONS   DEV=&_outputdevice ;
            ODS LISTING CLOSE;
                ODS HTML PATH="&outputDirectory"       (URL=NONE)
                BODY='max_age_list_cls.htm'
                NOGTITLE NOGFOOTNOTE
                STYLE=&_outputStyle._prt   ;
            TITLE1  "&title1_closed &title1_descending";
            TITLE2  "&limit_cls &title2_of &num_cls &title2_alerts";
            proc print data=work.max_cls_alerts label Split='_' NOOBS;
                VAR &Varlist1 &Varlist2 &Varlist3 &Varlist4 ;
            RUN;
            ODS _ALL_ CLOSE;
            ODS LISTING;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to generate output detail report: MAX_CLS_ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
        %end;

        /* ------------------------------------------------------------------ */
        /* MAX_SUP_ALERTS                                                     */
        /* ------------------------------------------------------------------ */
        %if  &actual_number_sup ne 0 %then  %do;
            OPTIONS   DEV=&_outputdevice ;
            ODS LISTING CLOSE;
            ODS HTML PATH="&outputDirectory"       (URL=NONE)
                BODY='max_age_list_sup.htm'
                NOGTITLE NOGFOOTNOTE
                STYLE=&_outputStyle._prt   ;
            TITLE1  "&title1_suppressed &title1_descending";
            TITLE2  "&limit_sup &title2_of &num_sup &title2_alerts";
            proc print data=work.max_sup_alerts label Split='_' NOOBS;
                VAR &Varlist1 &Varlist2 &Varlist3 &Varlist4 ;
            RUN;
            ODS _ALL_ CLOSE;
            ODS LISTING;
            %aml_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _amlmsg_= NOTE: AML: Unable to generate output detail report: MAX_SUP_ALERTS;
                %put_aml_error;
                %let error_sw =1;
                %return ;
            %end;
        %end;

    /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    %mend aml_create_age_reports;



    /* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv */
    /* ---------------------------------------------------------- */
    /*               MAIN PROCESSING MACRO                        */
    /*                 ALERT_AGE_REPORT                           */
    /* ---------------------------------------------------------- */
    /* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv */


    /* Set Module Name */
    %LOCAL _AMLMODULE_;
    %let _AMLMODULE_=ALERT_AGE_REPORT;

    /* Call preALERT_AGE_REPORT stub macro */
    %execute_macro(pre&_amlmodule_);

    /* Set error code to 0 and message to blank*/
    %let trans_rc = 0;
    %let _amlmsg_=;

    /* Set error switch - used in called macro routines to indicate error return code*/
    %global error_sw;
    %let error_sw = 0;

    /* Get title from xml file*/
    %label_maker(basefile=alert_age_meta
        ,table_name=REPORT_TITLE_MACRO_META
        ,generate=MACVAR) ;

    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* THIS SECTION CHECKS VALUES OF REPORT MACRO PARMS AND SETS DEFAULTS */
    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    %global _outputdevice;
    %let _outputdevice =javaimg;
    %IF %SYMEXIST(outputdevice) = 1  %then %do ;  /* Does variable exist?    */
        %IF &outputdevice ne %then %do ;          /* Does it have a value?   */
            %let _outputdevice = &outputdevice;    /* Then set value          */
        %end;
    %end;

    %let _title = Status of Alerts ;     /* OPTIONAL MACRO VARIABLE - set default */
    %IF %SYMEXIST(title) = 1  %then %do ;  /* Does variable exist?    */
        %IF &title ne %then %do ;          /* Does it have a value?   */
            %let _title = &title;          /* Then set value          */
        %end;
    %end;

    %let _display_limit = 1000 ;     /* OPTIONAL MACRO VARIABLE - set default */
    %IF %SYMEXIST(display_limit) = 1  %then %do ;  /* Does variable exist?    */
        %IF &display_limit ne %then %do ;          /* Does it have a value?   */
            %let _display_limit = &display_limit;          /* Then set value          */
        %end;
    %end;
    %if %sysfunc(notdigit(&_display_limit)) ne 0 %then %do ;
        %let _amlmsg_= NOTE: AML: Display_limit parameter not numeric. Value of &_display_limit ;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    %let _chart_report_YN = N;                        /* OPTIONAL MACRO VARIABLE - set default */
    %IF %SYMEXIST(chart_report_YN) = 1  %then %do ;  /* Does variable exist?    */
        %IF &chart_report_YN ne %then %do ;          /* Does it have a value?   */
            %if ( %upcase(&chart_report_YN) = Y or
                  %upcase(&chart_report_YN) = N )  %then %do ;
                %let _chart_report_YN = %upcase(&chart_report_YN);     /* Then set value          */
            %end;
        %end;
    %end;

    %let _chart_type = VBAR3D;                    /* OPTIONAL MACRO VARIABLE - set default */
    %let parm_string = VBAR HBAR BLOCK;
    %if %SYMEXIST(chart_type) = 1  %then %do ;  /* Does variable exist?    */
        %if &chart_type ne %then %do ;          /* does it have a value?   */
            %if  %index(&parm_string,%upcase(&chart_type)) %then %do;
                 %if %upcase(&chart_type) = VBAR  %then %let _chart_type=VBAR3D;
                 %if %upcase(&chart_type) = HBAR  %then %let _chart_type=HBAR3D;
                 %if %upcase(&chart_type) = BLOCK  %then %let _chart_type=BLOCK;
            %end;
        %end;
    %end;

    %let _bar_shape = block;                    /* OPTIONAL MACRO VARIABLE - set default */
    %IF %SYMEXIST(bar_shape) = 1  %then %do ;  /* Does variable exist?    */
        %IF &bar_shape ne %then %do ;          /* Does it have a value?   */
            %let _bar_shape =  &bar_shape;       /* Then set value          */
        %end;
    %end;

    %let _color1 =  ;                        /* OPTIONAL MACRO VARIABLE - set default */
    %IF %SYMEXIST(color1) = 1  %then %do ;  /* Does variable exist?    */
        %IF &color1 ne %then %do ;          /* Does it have a value?   */
            %let _color1 = Pattern1 color = &color1 ;       /* Then set value          */
        %end;
    %end;

    %let _color2 =  ;                        /* OPTIONAL MACRO VARIABLE - set default */
    %IF %SYMEXIST(color2) = 1  %then %do ;  /* Does variable exist?    */
        %IF &color2 ne %then %do ;          /* Does it have a value?   */
            %let _color2 = Pattern2 color = &color2 ;       /* Then set value          */
        %end;
    %end;

    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

    /* ------------------------------------------------------------------ */
    /* Generate Macros                                                    */
    /* ------------------------------------------------------------------ */
    %label_maker(basefile=alert_age_meta
            ,table_name=ALERT_AGE_FORMAT_MACRO_META
            ,generate=MACVAR) ;
    %label_maker(basefile=alert_age_meta
            ,table_name=ALERT_AGE_AGE_LABELS_META
            ,generate=MACVAR) ;
    %label_maker(basefile=alert_age_meta
            ,table_name=ALERT_AGE_REPORT_MACRO_META
            ,generate=MACVAR) ;

    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* THIS SECTION CREATES FORMATS FOR REPORTS
    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    PROC FORMAT;
    value  $rptstat 'ACT'="&days_act"
                    'CLS'="&days_cls"
                    'SUP'="&days_sup"
                    other= "&other_value";
    run;

    PROC FORMAT;
    value  $rpttab  'ACT'="&num_act"
                    'CLS'="&num_cls"
                    'SUP'="&num_sup"
                    other= "&other_value";
    run;

    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* THIS SECTION COUNTS NUMBER OF ALERTS                                   */
    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

    /* ---------------------------------------------------------- */
    /*      COUNT OF RECORDS BY STATUS CODE                       */
    /* ---------------------------------------------------------- */
    %if %sysfunc(exist(SEG_KC.fsk_alert)) %then %do;
        proc sql  noprint;
            create table work.preliminary_status_count   as
            select   alert_status_code,
                count(alert_status_code) as count_Status_code
            from SEG_KC.fsk_alert
            group by alert_status_code
            order by alert_status_code;
        quit;
        %aml_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _amlmsg_= NOTE: AML: SQL error creating PRELIMINARY_STATUS_COUNT;
            %put_aml_error;
            %goto MACRO_END;
        %end;
    %end;
    %else %do;
        %let _amlmsg_= NOTE: AML: SEG_KC.FSK_ALERT does not exist or is not accessible;
        %put_aml_error;
        %goto MACRO_END;
    %end;
    %let dsid=%sysfunc(open(work.preliminary_status_count,i));
    %let num_obs=%sysfunc(attrn(&dsid,nobs));
    %let rc=%sysfunc(close(&dsid));
    %if &num_obs = 0 %then %do;
        %let _amlmsg_= NOTE: AML: Zero records selected for this report, no report will be produced;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    /* ---------------------------------------------------------- */
    /*      LIST OF VALID STATUS CODES FROM LOV TABLE             */
    /* ---------------------------------------------------------- */
    proc sql  noprint;
        create table work.status_list  as
        select lov_type_code as alert_status_code
        from SEG_KC.fsk_lov
        where lov_type_name= "ALERT STATUS"
        order by lov_type_code;
    quit;
    %aml_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: SQL error creating STATUS_LIST;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    /* ---------------------------------------------------------- */
    /*      LIST OF ALL STATUS CODES MATCHED WITH COUNT           */
    /* ---------------------------------------------------------- */
    proc sql  noprint;
         create table work.status_all_count as
         select status_list.alert_status_code format=$3.,
             status_count.count_status_code
         from work.status_list as status_list left join work.preliminary_status_count as status_count
         on (status_list.alert_status_code = status_count.alert_status_code)
         order by alert_status_code;
         quit;
    %aml_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: SQL error creating STATUS_ALL_COUNT;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    /* ---------------------------------------------------------- */
    /*      Create macro variables for all alert status codes     */
    /* ---------------------------------------------------------- */
    data  work.status_all_count;
        set work.status_all_count ;
            if count_status_code = .  then count_status_code = 0;
            call symput("NUM_" || trim(left(alert_status_code)) ,trim(left(put(count_status_code,8.))));
            if count_status_code gt &_display_limit then report_limit = &_display_limit;
            else report_limit = count_status_code;
            call symput("LIMIT_" || trim(left(alert_status_code)) ,trim(left(put(report_limit,8.))));
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to read table: STATUS_ALL_COUNT;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* END SECTION COUNTING NUMBER OF ALERTS                                  */
    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
    /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

    /* ---------------------------------------------------------- */
    /* PROCESS ACTIVE Alerts                                      */
    /* ---------------------------------------------------------- */
    %let alert_type=ACT;
    %let event_code_value=ACT;

    %aml_get_active_alerts;
    %if &error_sw ne 0 %then %goto MACRO_END;

    /* ---------------------------------------------------------- */
    /* PROCESS CLOSED Alerts                                      */
    /* ---------------------------------------------------------- */
    %let alert_type=CLS ;
    %let event_code_value=CLS;

    %aml_get_alert_events;
    %if &error_sw ne 0 %then %goto MACRO_END;

    /* ---------------------------------------------------------- */
    /* PROCESS SUPPRESSED Alerts                                  */
    /* ---------------------------------------------------------- */
    %let alert_type=SUP;
    %let event_code_value=SUP;

    %aml_get_alert_events;
    %if &error_sw ne 0 %then %goto MACRO_END;

    /* ---------------------------------------------------------- */
    /* This macro generates the code necessary to create a        */
    /* summary file by status code grouped by Average and         */
    /* maximum values along with associated HTML links            */
    /* ---------------------------------------------------------- */

    %aml_generate_summary_file;
    %if &error_sw ne 0 %then %goto MACRO_END;

    /* ---------------------------------------------------------- */
    /* EXECUTE GENERATED CODES                                    */
    /* ---------------------------------------------------------- */
    %include tempcode; /* Execute generated code         */
    run;

    /* ---------------------------------------------------------- */
    /* Create Chart and Table                                     */
    /* ---------------------------------------------------------- */
    %aml_create_age_charts ;
    %if &error_sw ne 0 %then %goto MACRO_END;

    /* ---------------------------------------------------------- */
    /* Create Linked to Reports                                   */
    /* ---------------------------------------------------------- */
    %let Varlist1 = alert_age alert_id alert_status_code alert_description ;
    %let Varlist2 = primary_entity_level_code primary_entity_number primary_entity_name;
    %let Varlist3 = money_laundering_risk_score  terror_financing_risk_score;
    %let Varlist4 = create_date  scenario_name actual_values_text create_user_id;


    %aml_create_age_reports;
    %if &error_sw ne 0 %then %goto MACRO_END;

    /* ---------------------------------------------------------- */
    /*  GENERATE NO RECORD FOUND HTML PAGE                        */
    /* ---------------------------------------------------------- */

    %no_record_htm(Msg1 =%superq(No_alert_message),
         msg2=,
         msg3=);


    /* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv */
    /* ---------------------------------------------------------- */
    /*            END OF MAIN PROCESSING MACRO                    */
    /*                 ALERT_AGE_REPORT                           */
    /* ---------------------------------------------------------- */
    /* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv */


%MACRO_END:   /*  EXIT MACRO */
    %label_maker(basefile=alert_age_meta,
                    table_name=ALERT_AGE_FORMAT_MACRO_META,
                    generate=MACDEL) ;
    /* Delete title macro variable */
    %label_maker(basefile=alert_age_meta
                    ,table_name=REPORT_TITLE_MACRO_META
                    ,generate=MACDEL) ;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
       %execute_macro(amlerror);
    %else %do;
       %execute_macro(post&_amlmodule_);
       %execute_macro(amlsuccess);
    %end;

%mend alert_age_report;
