/******************************************************************************/
/*                  S A S   S A M P L E   L I B R A R Y                       */
/*                                                                            */
/*    NAME: agp.sas                                                           */
/*   TITLE: Alert Generation Process                                          */
/* PRODUCT: SAS Anti-Money Laundering                                         */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/******************************************************************************/

/* ************************************************************************ */
/*  Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA       */
/*  ---All Rights Reserved.                                                 */
/* ************************************************************************ */

/******************************************************************************/
/*                                                                            */
/*       Host and Site specific customizations may be REQUIRED                */
/*                                                                            */
/******************************************************************************/

%macro aml_agp;

   /*------------------------------------------------------------------
    * Make sure the AML start datetime has been set before proceeding.
    *------------------------------------------------------------------*/
   %aml_start_datetime_read;
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
   %end;

   /*------------------------------------------------------------------
    * Store start datetime of the AML AGP process in a data set.
    *------------------------------------------------------------------*/
   %aml_start_datetime_init;
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
   %end;

/*************************** Scenario Processing ******************************/

/******************************************************************************/
/* Note: When parameters are updated for a scenario, the code must be         */
/*       re-generated and the Alert Generation Process re-executed.           */
/*       If re-executing for the same RUNDATE in the same sessions, then      */
/*       be certain to clear the STG_ALER directory before executing.         */
/*       Otherwise the STG_ALER data sets will contain duplicate alerts.      */
/*       Set get_scenario_performance_stats to Y in the autoexec so           */
/*       scenariocodegen generates code to gather performance statistics      */
/*       for scenarios.                                                       */
/******************************************************************************/
   %scenariocodegen(outLib=STG_ALER,
                    outDS=generated_headers);
   
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %goto MACRO_END;
   %end;

   %aml_run_generated_headers(inLib=STG_ALER,
                              inDS=generated_headers,
                              exclude_header=empty_header);
   
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %goto MACRO_END;
   %end;

   /*-------------------------------------------------
    * An example of how to include other headers not
    * processed by today's run of scenariocodegen and
    * aml_run_generated_headers:
    *-------------------------------------------------*/
   /*
   %aml_section_header(Section=ANOTHER_HEADER);
      %include '!AMLROOT/scenario/codegen/another_header.sas';
      %aml_rcset(&syserr)
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _AMLMODULE_=ANOTHER_HEADER;
         %let _amlmsg_=ERROR: AML: another_header.sas failed check log.;
         %put_aml_error;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=ANOTHER_HEADER);
   */

/**************************** Alert Processing ********************************/

   /* Combine alerts */
   %aml_section_header(Section=AGP_COMBINE_ALERTS);
      %combine_alerts(inlib=STG_ALER,
                      outLib=STG_ALER,
                      outDS=combined_alerts,
                      outRiskLib=STG_ALER,
                      outRiskDS=risk_factor_alerts);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_COMBINE_ALERTS);

/* Test for Zero Alerts generated if no alerts end process. */
   %if %sysfunc(exist(STG_ALER.combined_alerts)) %then %do;
      %let dsid = %sysfunc(open(STG_ALER.combined_alerts));
      %let nobs = %sysfunc(attrn(&dsid, nobs));
      %let rc = %sysfunc(close(&dsid));
      %if &nobs eq 0 %then %do;
         %let _amlmsg_=NOTE: AML: No Alerts generated for &runasofdate..;
         %put_aml_error;
         %goto MACRO_END;
      %end;
   %end;
   %else %goto MACRO_END; /* Test for Zero Alerts */

   /* Risk Ranking */
   %aml_section_header(Section=AGP_RISKRANKING);
      %riskranking(inLib=STG_ALER,
                   inDS=combined_alerts,
                   inRiskLib=STG_ALER,
                   inRiskDS=risk_factor_alerts,
                   outLib=STG_ALER,
                   outDS=ranked_alerts,
                   outRiskLib=MST_ALER,
                   outRiskDS=ranked_risk_factor_alerts,
                   riskonly=Y,
                   %if %upcase(&dbkcflavor) eq TERADATA %then %do;
                    bl_temp=NO,
                   %end;
                   %else %do;
                    bl_temp=YES,
                   %end;
                   bl_tempload=);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_RISKRANKING);

   /* Suppression */
   %aml_section_header(Section=AGP_SUPPRESSION);
      %suppression(inLib=STG_ALER,
                   inDS=ranked_alerts,
                   outLib=STG_ALER,
                   outDS=suppressed_alerts);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_SUPPRESSION);

   /* Get entity information */
   %aml_section_header(Section=AGP_GET_ENTITY_INFO);
      %get_entity_info(inLIB=STG_ALER,
                       inDS=suppressed_alerts,
                       outLIB=STG_ALER,
                       outDS=entity_alerts,
                       %if %upcase(&dbkcflavor) eq TERADATA %then %do;
                        bl_temp=NO,
                       %end;
                       %else %do;
                        bl_temp=YES,
                       %end;
                       bl_tempload=);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_GET_ENTITY_INFO);

   /* Routing */
   %aml_section_header(Section=AGP_ROUTE);
      %route(inLib=STG_ALER,
             inDS=entity_alerts,
             outLib=MST_ALER,
             outDS=routed_alerts);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_ROUTE);

   /* Make the UI unavailable to users */
      %set_ui_availability(availability_ind=N);

   /* Load alerts into the KC */
   %aml_section_header(Section=AGP_LOADALERTS);
      %loadAlerts(routedAlertLib=MST_ALER,
                  routedAlertTable=routed_alerts);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_LOADALERTS);

   /* Update Alert History */
   %aml_section_header(Section=AGP_UPDATE_ALERT_HISTORY);
      %update_alert_history(inLib=MST_ALER,
                            inDS=routed_alerts,
                            bulkload=);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_UPDATE_ALERT_HISTORY);

   /* Load risk factors into the KC */
   %aml_section_header(Section=AGP_LOADRISKFACTORS);
      %loadRiskFactors(riskFactorLib=MST_ALER,
                       riskFactorTable=ranked_risk_factor_alerts);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_LOADRISKFACTORS);

   /* Create replication temporary tables */
   /* %get_replicate_info must be run before
      %replicateTransactions and %replicateProfile */
   %aml_section_header(Section=AGP_GET_REPLICATE_INFO);
      %get_replicate_info(routedAlertLib=MST_ALER,
                          routedAlertTable=routed_alerts,
                          %if %upcase(&dbkcflavor) eq TERADATA %then %do;
                           bl_temp=NO,
                          %end;
                          %else %do;
                           bl_temp=YES,
                          %end;
                          bl_tempload=);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_GET_REPLICATE_INFO);

   /* Replicate transactions for FSC_ACCOUNT_EVENT_FACT */
   %aml_section_header(Section=AGP_REPLICATE_ACCOUNT_EVENT);
      %replicateTransactions(routedAlertLib=MST_ALER,
                             routedAlertTable=routed_alerts,
                             coreFactTable=fsc_account_event_fact,
                             kcFactTable=fsk_account_event_alert);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_REPLICATE_ACCOUNT_EVENT);

   /* Replicate transactions for FSC_ACCOUNT_TRADE_FACT */
   %aml_section_header(Section=AGP_REPLICATE_ACCOUNT_TRADE);
      %replicateTransactions(routedAlertLib=MST_ALER,
                             routedAlertTable=routed_alerts,
                             coreFactTable=fsc_account_trade_fact,
                             kcFactTable=fsk_account_trade_alert,
                             date_key=%str(settlement_date_key));
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_REPLICATE_ACCOUNT_TRADE);

   /* Replicate transactions for FSC_CASH_FLOW_FACT */
   %aml_section_header(Section=AGP_REPLICATE_CASH_FLOW);
      %replicateTransactions(routedAlertLib=MST_ALER,
                             routedAlertTable=routed_alerts,
                             coreFactTable=fsc_cash_flow_fact,
                             kcFactTable=fsk_cash_flow_alert);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_REPLICATE_CASH_FLOW);

   /* Replicate profile for FSC_ACCOUNT_PROFILE_FACT */
   %aml_section_header(Section=AGP_REPLICATE_ACC_PROFILE);
      %replicateProfile(routedAlertLib=MST_ALER,
                        routedAlertTable=routed_alerts,
                        coreFactTable=fsc_account_profile_fact,
                        kcFactTable=fsk_account_profile_alert,
                        bulkload=);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_REPLICATE_ACC_PROFILE);

   /* Replicate profile for FSC_PARTY_PROFILE_FACT */
   %aml_section_header(Section=AGP_REPLICATE_PTY_PROFILE);
      %replicateProfile(routedAlertLib=MST_ALER,
                        routedAlertTable=routed_alerts,
                        coreFactTable=fsc_party_profile_fact,
                        kcFactTable=fsk_party_profile_alert,
                        bulkload=);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_REPLICATE_PTY_PROFILE);

   /* Make the UI available to users */
     %set_ui_availability(availability_ind=Y);

   *Send Manager e-Mail;
   %aml_section_header(Section=AGP_NOTIFY_MANAGER);
      %notify_manager(age=30,
                      delimtype=CSV);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_NOTIFY_MANAGER);

   *Send Reminder e-Mail;
   %aml_section_header(Section=AGP_SEND_ALERT_REMINDER);
      %send_alert_reminder;
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
      %end;
   %aml_section_footer(Section=AGP_SEND_ALERT_REMINDER);

   /*------------------------------------------------------------------
    * Clear the start datetime of the AML AGP process.
    *------------------------------------------------------------------*/
   %aml_start_datetime_clear;
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
   %end;

   %MACRO_END:

      %if &trans_rc eq 3 or &trans_rc ge 5 %then
         %execute_macro(amlerror);
      %else %do;
         %execute_macro(amlsuccess);
      %end;

%mend;
