/******************************************************************************/
/*                  S A S   S A M P L E   L I B R A R Y                       */
/*                                                                            */
/*    NAME: aml_ca_execute_classifiers.sas                                    */
/*   TITLE: Compliance Analytics Processing                                   */
/* PRODUCT: SAS Anti-Money Laundering                                         */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/*    DESC: Run Compliance Analytics alert classifiers                        */
/*                                                                            */
/* Pre-Reqs:                                                                  */
/*   Account and Party signatures created.                                    */
/*                                                                            */
/* Outputs:                                                                   */
/*   Data set of scored alerts.                                               */
/*----------------------------------------------------------------------------*/
/* USAGE: %aml_ca_execute_classifier(                                         */
/*             classifierPath= location of classifiers                        */
/*             caStg= Stage Compliance Analytics libname                      */
/*             caMst= Master Compliance Analytics libname                     */
/*             inStgAccSigDS= Input Stage Account Signature data set          */
/*             inStgPtySigDS= Input Stage Party Signature data set            */
/*             outMstScoredAlertsDS= master data set of scored alerts         */
/*             scoreCutoff= score used for suppression 0-1000                 */
/*             fpPassthroughPct= percent of below 500 scores to not suppress  */
/*             brInd= Run account customer business rules?(Y/N)               */
/*             updateDB= Update FSK_ALERT, FSK_ALERT_EVENT,                   */
/*                        FSK_ALERT_COMMENT (Y/N)                             */
/******************************************************************************/

/* ********************************************************************** */
/*  Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA     */
/*  ---All Rights Reserved.                                               */
/* ********************************************************************** */

%macro aml_ca_execute_classifiers(classifierPath=&AMLROOT/classifier/ca,
                                  caStg=stg_ca,
                                  caMst=mst_ca,
                                  inStgAccSigDS=scoreSigAccAll,
                                  inStgPtySigDS=scoreSigPtyAll,
                                  outMstScoredAlertsDS=scoredAlertsAll,
                                  scoreCutoff=500,
                                  fpPassthroughPct=5,
                                  brInd=N,
                                  updateDB=N) ;

 %put NOTE: {START: aml_ca_execute_classifiers};

 /*--------------------------------------------------------
  * Make sure the AML start datetime has been set.
  * Assign its value to macro variable aml_start_datetime.
  *--------------------------------------------------------*/
 %aml_start_datetime_read;
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
   %let bat_abort=Y;
   %goto exit;
 %end;

 /********************************************************/
  %let step=run account level classifiers;
 /*********************************************************/
 data &caStg..class_1001;
  set &caStg..&inStgAccSigDS(where=(classifier_id=1001));
   %include "&classifierPath./class_1001.sas";
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 proc append base=&caStg..scoredAccBase
             data=&caStg..class_1001 force;
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 data &caStg..class_1002;
  set &caStg..&inStgAccSigDS(where=(classifier_id=1002));
   %include "&classifierPath./class_1002.sas";
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 proc append base=&caStg..scoredAccBase
             data=&caStg..class_1002 force;
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 data &caStg..class_1003;
  set &caStg..&inStgAccSigDS(where=(classifier_id=1003));
   %include "&classifierPath./class_1003.sas";
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 proc append base=&caStg..scoredAccBase
             data=&caStg..class_1003 force;
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;


 data &caStg..class_unknown_acc;
  length EM_CLASSIFICATION $%dmnorlen;
  set &caStg..&inStgAccSigDS(where=(classifier_id=-1));
   EM_EVENTPROBABILITY = -1;
   EM_CLASSIFICATION = '';
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 proc append base=&caStg..scoredAccBase
             data=&caStg..class_unknown_acc force;
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 data &caStg..scoredAccBase;
  set &caStg..scoredAccBase;
   if classifier_id eq -1 then do;
     fpPTind = 'N';
     rsSupressInd = 'N';
     suppressInd = 'N';
   end;
  run;   

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 /********************************************************/
  %let step=run Party level classifiers;
 /*********************************************************/

 data &caStg..scoredPtyBase;
  set &caStg..&inStgPtySigDS(where=(classifier_id=2001));
   %include "&classifierPath./class_2001.sas";
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 data &caStg..class_2002;
  set &caStg..&inStgPtySigDS(where=(classifier_id=2002));
   %include "&classifierPath./class_2002.sas";
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 proc append base=&caStg..scoredPtyBase
             data=&caStg..class_2002 force;
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 data &caStg..class_unknown_pty;
  length EM_CLASSIFICATION $%dmnorlen;
  set &caStg..&inStgPtySigDS(where=(classifier_id=-1));
   EM_EVENTPROBABILITY = -1;
   EM_CLASSIFICATION = '';
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 proc append base=&caStg..scoredPtyBase
             data=&caStg..class_unknown_pty force;
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 data &caStg..scoredPtyBase;
  set &caStg..scoredPtyBase;
   if classifier_id eq -1 then do;
     fpPTind = 'N';
     rsSupressInd = 'N';
     suppressInd = 'N';
   end;
  run;   

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 /********************************************************/
  %let step=create view with all scored alerts;
 /*********************************************************/
 proc append base=&caStg..scoredBase
             data=&caStg..scoredAccBase force;
  run;

 data &caStg..scoredBase;
  set &caStg..scoredBase;
   attrib party_key               length=8   format=11.
          party_number            length=$50 format=$50.
          party_type_desc         length=$20 format=$20.
          aggregate_assets_amount length=8   format=20.5
          number_of_accounts      length=8   format=8.
          tran                    length=8   format=8.
          ptyDim                  length=8   format=8.
          ptyProf                 length=8   format=8.
          nba                     length=8   format=8.
        %if &aml_account_cc_classifier_exists = 1 or
            &aml_party_cc_classifier_exists = 1 %then %do;
          ccy2_flag               length=8   format=8.
          ccy3_flag               length=8   format=8.
          ccy4_flag               length=8   format=8.
          ccy5_flag               length=8   format=8.
          ccy6_flag               length=8   format=8.
          ccyCnt_2                length=8   format=11.
          ccyCnt_3                length=8   format=11.
          ccyCnt_4                length=8   format=11.
          ccyCnt_5                length=8   format=11.
          ccyCnt_6                length=8   format=11.
        %end;
          ;
  run;

 proc append base=&caStg..scoredBase
             data=&caStg..scoredPtyBase force nowarn;
  run;

 proc sort data=&caStg..scoredBase;
   by classifier_id;
  run;

 data &caStg..ScoredAlerts_&runasofdate;
   merge &caStg..scoredBase(in=a) &caMst..classifierflag(in=b);
   by classifier_id;
   if a and b;
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 data &caStg..EnhancedScoredAlerts_&runasofdate;
   attrib classifier_decision  length=$50 format=$50.
          classifier_score     length=8   format=8.
          suppressInd          length=$1  format=$1.
          rsSupressInd         length=$1  format=$1.
          bRulePTInd           length=$1  format=$1.
          ;

   set &caStg..ScoredAlerts_&runasofdate;

   bRulePTInd='N';  /*customer can set to 'Y' in their
                      business rules pass-through file read below*/
   classifier_decision = em_classification;
   classifier_score = round(em_eventprobability * 1000,1);

   if classifier_score ne . then do;
     if classifier_score GE &scoreCutoff then do;
       suppressInd = 'N';
       rsSupressInd = 'N';
     end;
     else do;
       suppressInd = 'Y';
       rsSupressInd = 'Y';
     end;
   end;

   %if &brInd eq Y %then %do;
     %include "&classifierPath./BusinessRules.sas";
   %end;
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 /********************************************************/
  %let step=allow percentage of False Positives through;
 /*********************************************************/
 %let sampleSize=0;
 proc sql noprint;
   select round(count(*)* (&fpPassthroughPct/100)) into: sampleSize
     from &caStg..EnhancedScoredAlerts_&runasofdate(where=(suppressInd='Y'));
 quit;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 %if &sampleSize gt 0 %then %do;
   proc surveyselect
     data=&caStg..EnhancedScoredAlerts_&runasofdate(where=(suppressInd='Y'))
     out=work.tmpSampledFP(keep=alert_id)
     method=srs
     sampsize=&sampleSize.;
    run;

   %aml_chkrc;
   %if &bat_abort=Y %then %do;
     %put NOTE: STEP: &step;
     %goto exit;
   %end;
 %end;
 %else %do;
   data work.tmpSampledFP;
     attrib alert_id length=8;
     stop;
    run;
 %end;

 proc sort data=&caStg..EnhancedScoredAlerts_&runasofdate;
   by alert_id;
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 proc sort data=work.tmpSampledFP;
   by alert_id;
  run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 data &caMst..&outMstScoredAlertsDS;
   attrib fpPTInd format=$1. length=$1;
   merge &caStg..EnhancedScoredAlerts_&runasofdate(in=inAll)
         work.tmpSampledFP(in=inFP);
   by alert_id;
   if inAll and inFP then do;
     fpPTInd = 'Y';
     suppressInd='N';
     output;
   end;
   else if inAll and not inFP then do;
     fpPTInd = 'N';
     output;
   end;
 run;

 %aml_chkrc;
 %if &bat_abort=Y %then %do;
   %put NOTE: STEP: &step;
   %goto exit;
 %end;

 %let num_comments_events = 0;

 %if &updateDB = Y %then %do;
   %put NOTE: Create alert event and comment.  FSK_ALERT_EVENT and FSK_ALERT_COMMENT = &updateDB;

   /*-----------------------------------------------------------------------------
    * These are all of the alerts that we want to create comments and events for
    *-----------------------------------------------------------------------------*/
   data &scoredAlertsDS._&runasofdate(keep=alert_id);
    set &caMst..&outMstScoredAlertsDS(where=(loadyn='Y' and
                                             suppressInd='Y'));
    run;
 
   /************************************************/
    %let step=Create new alert comment records;
   /************************************************/
   data work.tmpAlertComment;     
     if 1 eq 0 then set seg_kc.fsk_alert_comment(drop=alert_id);
     set &scoredAlertsDS._&runasofdate;
                   /* ca_alert_comment_text = Suppressed due to Compliance Analytics */
     alert_comment_text = sasmsg('sashelp.amlmsg', 'ca_alert_comment_text', 'NOQUOTE');
     item_id = .;
     create_date = &aml_start_datetime;
     create_user_id = "&SYSUSERID";
     logical_delete_ind = 'N';
     segment_id = "&segKCDBUser";
     /*comment_id set later*/
    run;

   %aml_chkrc;
   %if &bat_abort=Y %then %do;
     %put NOTE: STEP: &step;
     %goto exit;
   %end;

   /*********************************************/
    %let step=Create new alert event records;
   /*********************************************/
   data work.tmpAlertEvent;       
     if 1 eq 0 then set seg_kc.fsk_alert_event(drop=alert_id);
     set &scoredAlertsDS._&runasofdate;
     event_type_code = 'SUP';
                  /* ca_alert_event_description = Suppressed due to Compliance Analytics */
     event_description = sasmsg('sashelp.amlmsg', 'ca_alert_event_description', 'NOQUOTE');
     create_date = &aml_start_datetime;
     create_user_id = "&SYSUSERID";
     segment_id = "&segKCDBUser";
     /*event_id set later*/
    run;

   %aml_chkrc;
   %if &bat_abort=Y %then %do;
     %put NOTE: STEP: &step;
     %goto exit;
   %end;

   /***********************************/
    %let step=Count suppressed alerts;
   /***********************************/
   
   /*----------------------------------------------
    * We need to add this many comments and events
    *----------------------------------------------*/
   %let num_comments_events = %aml_get_observation_count(table=&scoredAlertsDS._&runasofdate, sasdataset=Y);

   %if &num_comments_events eq -1 %then %do;
     %put NOTE: STEP: &step;
     %put NOTE: WORK.&scoredAlertsDS._&runasofdate does not exist.  Probable cause: No alerts suppressed.;
     %goto exit;
   %end;
 
 %end;
 
 /***********************************************************************/
  %let step=Get sequence numbers for and add alert comments and events;
 /***********************************************************************/
 
 %if &num_comments_events gt 0 and &updateDB = Y %then %do;
   /*------------------------------------------
    * First get sequence numbers for comments
    *------------------------------------------*/
   %aml_get_sequence_ids(numIds=&num_comments_events);
   
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto exit;
   %end;
   
   data &caMst..&outMstScoredAlertsDS._Cmts;
    set work.tmpAlertComment;
     comment_id = symgetn('seqId'||(strip(put(_n_,best32.))));
   run;

   %aml_chkrc;
   %if &bat_abort=Y %then %do;
     %put NOTE: STEP: &step;
     %goto exit;
   %end;
   
   proc append base=seg_kc.fsk_alert_comment
               data=&caMst..&outMstScoredAlertsDS._Cmts
               force;
    run;

   %aml_chkrc;
   %if &bat_abort=Y %then %do;
     %put NOTE: STEP: &step;
     %goto exit;
   %end;

   /*---------------------------------------
    * Next get sequence numbers for events
    *---------------------------------------*/
   %aml_get_sequence_ids(numIds=&num_comments_events);
   
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto exit;
   %end;
   
   data &caMst..&outMstScoredAlertsDS._Evts;
    set work.tmpAlertEvent;
     event_id = symgetn('seqId'||(strip(put(_n_,best32.))));
   run;

   %aml_chkrc;
   %if &bat_abort=Y %then %do;
     %put NOTE: STEP: &step;
     %goto exit;
   %end;
   
   proc append base=seg_kc.fsk_alert_event
               data=&caMst..&outMstScoredAlertsDS._Evts
               force;
    run;

   %aml_chkrc;
   %if &bat_abort=Y %then %do;
     %put NOTE: STEP: &step;
     %goto exit;
   %end;
 %end;
 %else %do;
   %put NOTE: (aml_ca_execute_classifier) NO UPDATING OF FSK_ALERT_COMMENT OR FSK_ALERT_EVENT PERFORMED;
 %end; 


 /********************************************************/
  %let step=update FSK_ALERT;
 /*********************************************************/
 %if &num_comments_events gt 0 and &updateDB = Y %then %do;
   %put NOTE: Update FSK_ALERT = &updateDB;

   proc datasets library=&scoredAlertsLib;
     modify &scoredAlertsDS._&runasofdate;
     index create alert_id;
   quit;

   %aml_chkrc;
   %if &bat_abort=Y %then %do;
     %put NOTE: STEP: &step;
     %goto exit;
   %end;
 
   proc sql noprint;
     update seg_kc.fsk_alert
        set alert_status_code = 'SUP',
            alert_suppressed_ind = 'Y'
      where alert_id in
       (
        select alert_id
          from &caMst..&outMstScoredAlertsDS
         where suppressInd = 'Y'
           and loadyn = 'Y'
       );
   quit;

   %aml_chkrc;
   %if &bat_abort=Y %then %do;
     %put NOTE: STEP: &step;
     %goto exit;
   %end;
 %end;
 %else %do;
   %put NOTE: (aml_ca_execute_classifier) NO UPDATING OF FSK_ALERT PERFORMED;
 %end;

 %put NOTE: {END: aml_ca_execute_classifier};
 %let syscc=0;

 %execute_macro(postaml_ca_execute_classifiers);

 %exit:
 %if &bat_abort=Y %then %abort cancel 4;

%mend aml_ca_execute_classifiers;
