/******************************************************************************/
/*                  S A S   S A M P L E   L I B R A R Y                       */
/*                                                                            */
/*    NAME: dist_active_alerts_report.sas                                     */
/*   TITLE: Distribution of Active Alerts Report                              */
/* PRODUCT: SAS Anti-Money Laundering                                         */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/******************************************************************************/
/*  Description: Create reports showing distribution of active alerts over time
/*           (days). The distribution is grouped into periods.
/*
/* PARAMETERS:
/* <!-- ODS Parms -->
/*      outputDirectory = DIRECTORY FOR THIS REPORT
/*      outputBody = REPORT HTML BODY NAME:  body.htm
/*      outputDevice = GRAPHICS RENDERING:  javaimg/actximg
/*      _outputStyle = STYLE TO APPLY:  sasweb, banker etc
/* <!-- Report Parms -->
/*      num_groups = NUMBER OF MONTHS TO TREND: 10
/*      chart_type = type of chart vbar hbar
/*      title = REPORT TITLE
/*      bar_color = BAR_COLOR:  blue
/*      bar_shape = SHAPE OF BARS:  block
/*      table_report_YN>N = PRODUCE A AN APPENDED TABULATE REPORT:
/* <!-- Called routines and macros -->
/*      %aml_rcset
/*      %put_aml_error
/*      %aml_execute_command
/*      %no_record_htm
/*
/******************************************************************************/

/* ************************************************************************** */
/*  Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA         */
/*  ---All Rights Reserved.                                                   */
/* ************************************************************************** */

%macro dist_active_alerts_report ;

    /* Set Module Name */
    %local _AMLMODULE_;
    %let _AMLMODULE_= DIST_ACTIVE_ALERTS_REPORT;

    /* Call preDIST_ACTIVE_ALERTS_REPORT stub macro */
    %execute_macro(pre&_amlmodule_)

    /* Set error code to 0 and message to blank*/
    %let trans_rc = 0;
    %let _amlmsg_=;

   /*--------------------------------------------------------
    * Make sure the AML start datetime has been set.
    * Assign its value to macro variable aml_start_datetime.
    *--------------------------------------------------------*/
   %aml_start_datetime_read;
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
   %end;

    /* Get title from xml file*/
    %label_maker(basefile=dist_active_alerts_meta
            ,table_name=REPORT_TITLE_MACRO_META
            ,generate=MACVAR) ;

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*  SET REPORTING PARMS                                                       */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    data _null_;
    %let _num_groups =10;
    %IF %SYMEXIST(num_groups) = 1  %then %do ;  /* Does variable exist?   */
        %IF &num_groups ne . %then %do ;       /* Does it have a value?  */
           %let _num_groups = &num_groups;      /* Then set value            */
        %end;
    %end;

    %if &_num_groups gt 30 %then %do;
        %put  NOTE: (&sysmacroname)  The number groups selected,  %trim(%left(&_num_groups)),
    is greater than 30.  Recommended value is less than 30. Setting number of groups to 10.;
        %let _num_groups = 10;
    %end;

    %let _outputdevice =javaimg;
    %IF %SYMEXIST(outputdevice) = 1  %then %do ;  /* Does variable exist?    */
        %IF &outputdevice ne %then %do ;          /* Does it have a value?   */
           %let _outputdevice = &outputdevice;    /* Then set value          */
        %end;
    %end;

    %let _title = Distribution of Age of Active Alerts ;   /* OPTIONAL  - set default */
    %IF %SYMEXIST(title) = 1  %then %do ;           /* Does variable exist?    */
        %IF &title ne %then %do ;                   /* Does it have a value?   */
           %let _title =title1  &title;             /* Then set value          */
        %end;
    %end;

    %let _chart_type = VBAR3D;                    /* OPTIONAL MACRO VARIABLE - set default */
    %let parm_string = VBAR HBAR;
    %if %SYMEXIST(chart_type) = 1  %then %do ;  /* Does variable exist?    */
        %if &chart_type ne %then %do ;          /* does it have a value?   */
            %if  %index(&parm_string,%upcase(&chart_type)) %then %do;
                %if %upcase(&chart_type) = VBAR  %then %let _chart_type=VBAR3D;
                %if %upcase(&chart_type) = HBAR  %then %let _chart_type=HBAR3D;
            %end;
        %end;
    %end;

    %let _bar_shape = block;                    /* OPTIONAL MACRO VARIABLE - set default */
    %IF %SYMEXIST(bar_shape) = 1  %then %do ;  /* Does variable exist?    */
        %IF &bar_shape ne %then %do ;          /* Does it have a value?   */
           %let _bar_shape =  &bar_shape;      /* Then set value          */
        %end;
    %end;

    %let _bar_color=BLUE;                        /* OPTIONAL MACRO VARIABLE - set default */
    %IF %SYMEXIST(bar_color) = 1 %then %do ;    /* Does variable exist?    */
        %IF &bar_color ne   %then %do ;         /* Does it have a value?   */
           %let _bar_color=&bar_color;          /* Then set value          */
        %end;
    %end;

    %let _table_report_YN=Y;                           /* OPTIONAL MACRO VARIABLE - set default */
    %IF %SYMEXIST(table_report_YN) = 1 %then %do ;    /* Does variable exist?    */
        %IF &table_report_YN ne   %then %do ;         /* Does it have a value?   */
           %let _table_report_YN=%upcase(&table_report_YN); /* Then set value          */
        %end;
    %end;

    %label_maker(basefile=dist_active_alerts_meta
            ,table_name=DIST_ACTIVE_NODATA_MACROS_META
            ,generate=MACVAR) ;

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*  DETERMINE NUMBER OF ALERTS AND DATE RANGES
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* -------------------------------------------------------------------------- */
/* Get number of active alerts
/* -------------------------------------------------------------------------- */
    %if %sysfunc(exist(seg_kc.fsk_alert)) %then %do;
        proc sql  noprint;
            select (count(*)) into :number_active_alerts
            from seg_kc.fsk_alert as fsk_alert
            where fsk_alert.alert_status_code = 'ACT' ;
        quit;
        %aml_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _amlmsg_= NOTE: AML: Unable to access table: FSK_ALERT;
            %put_aml_error;
            %goto MACRO_END;
        %end;
        %if &number_active_alerts = 0 %then %do;
            %let _amlmsg_= NOTE: AML: There are no active alerts in the seg_kc.fsk_alert table.;
            %put_aml_error;
            %let _amlmsg_= NOTE: AML: The Distribution of Active Alerts report was not produced.;
            %put_aml_error;

            /* ---------------------------------------------------------- */
            /*  GENERATE NO RECORD FOUND HTML PAGE                        */
            /* ---------------------------------------------------------- */
            %no_record_htm(Msg1 =%superq(nodata_msg1),
                msg2=%superq(nodata_msg2),
                msg3=,
                htm_name= body);
            %goto MACRO_END;
        %end;
    %end;
    %else %do;
        %let _amlmsg_= NOTE: AML: seg_kc.FSK_ALERT does not exist or is not accessible;
        %put_aml_error;
        %goto MACRO_END;
    %end;
/* -------------------------------------------------------------------------- */
/* Get dates from alert table an count of active alerts                       */
/* -------------------------------------------------------------------------- */
    %if %upcase(&dbkcflavor) ne MYSQL %then %do;
        proc sql noprint;
           create table work.minmax_dates as
           select (min(fsk_alert.create_date)) format=dtdate9. as min_create,
           (max(fsk_alert.create_date)) format=dtdate9. as max_create
           from seg_kc.fsk_alert as fsk_alert
           where fsk_alert.alert_status_code = 'ACT'  ;
        quit;
        %aml_rcset(&sqlrc);
    %end;
    %else %do;
        proc sql;
            connect to &dbkcflavor as &dbkcflavor (user=&segKCDBUser
                           password=&segKCDBUserPwd &segKCDBConnOpts);
                create table work.minmax_dates as
                select min_create format=dtdate9., max_create format=dtdate9.
                from connection to &dbkcflavor
                (select cast(min(fsk_alert.create_date)as datetime) as min_create,
                       cast(max(fsk_alert.create_date)as datetime) as max_create
                from &segkcschema..fsk_alert as fsk_alert
                where fsk_alert.alert_status_code = 'ACT' );
            disconnect from &dbkcflavor;
        quit;
        %aml_rcset(&sqlrc);
    %end;
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to create table: MINMAX_DATES;
        %put_aml_error;
        %goto MACRO_END;
    %end;

/* -------------------------------------------------------------------------- */
/* Get date range and check whether _num_groups value is too large            */
/* -------------------------------------------------------------------------- */
    data _null_;
        set  work.minmax_dates;
        format  max_create   min_create  date9.;
        todays_date = datepart(&aml_start_datetime);
     /* Choose   either current date or max date to calculate charts */
        date_range=  todays_date-datepart(min_create)+1;
        call symput('max_create',put(todays_date,8.) );
     /*  or  */
          /* date_range= datepart(max_create)-datepart(min_create)+1;  */
          /* call symput('max_create',put(datepart(max_create),8.) );  */

        call symput('date_range',put(date_range,8.));
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to access table: MINMAX_DATES;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    data _null_;
        %put NOTE: (&sysmacroname)  The date range for this report is %left(&date_range) days. ;
        %if &_num_groups gt &date_range %then  %do;
            %put  NOTE: (&sysmacroname)  The number groups selected,  %trim(%left(&_num_groups)), is greater than the day range of active alerts, %trim(%left(&date_range)) ;
            %put  NOTE: (&sysmacroname)  The date range of active alerts will be used for the number of groups to distribute the alerts;
        %let _num_groups = &date_range;
        %end;
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*  BUILD FORMATS                                                             */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* -------------------------------------------------------------------------- */
/* Calculate intervals an for date range                                      */
/* -------------------------------------------------------------------------- */
    data work.range_list;
        incr = int(&date_range/&_num_groups);
        rem = mod(&date_range,&_num_groups);
        greater_than = &_num_groups - rem   ;
        start = 0 - incr;
        end = -1 ;
        do i = 1  to &_num_groups ;
            start + incr;
            end + incr  ;
            count = i;
            if i gt (greater_than +1) then start +1;
            if i gt greater_than  then end +1 ;
            range_label = trim(left(put(start,8.))) || '-' || trim(left(put(end,8.)));
            num =i  ;
            output work.range_list;
        end;
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to create table: RANGE_LIST;
        %put_aml_error;
        %goto MACRO_END;
    %end;
/* -------------------------------------------------------------------------- */
/* Create cntlin1 table for date age to range format                          */
/* Create cntlin2 table for range to number format                            */
/* Create cntlin3 table for number to range format                            */
/* Create template table of number and ranges                                 */
/* -------------------------------------------------------------------------- */
    data work.cntlin1  (keep=start  end   type  fmtname   hlo  label )
         work.cntlin2  (keep=start  end   type  fmtname   hlo  label )
         work.cntlin3  (keep=start  end   type  fmtname   hlo  label )
         work.template (keep=num label);

        type='N';
        hlo=' ';
        set work.range_list  end=last;
        /*  cntlin1   */
        fmtname='agerange';
        label=range_label;
        if last then hlo = 'h';
        output work.cntlin1;
        /*  cntlin2   */
        fmtname='rangenum';
        label=put(count,8.);
        if last then hlo = 'h';
        output work.cntlin2;
        /*  cntlin3   */
        fmtname='numrange';
        start=count;
        end=count;
        label=range_label;
        if last then hlo = 'h';
        output work.cntlin3;
        /*  template  */
        num=count;
        label=range_label;
        output work.template;
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to access table: RANGE_LIST;
        %put_aml_error;
        %goto MACRO_END;
    %end;
/* -------------------------------------------------------------------------- */
/* Create format to go to from age to range                                   */
/* Create format to go to from range to number                                */
/* Create format to go to from number to range                                */
/* -------------------------------------------------------------------------- */
    proc format cntlin=work.cntlin1;
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to access table: CNTLIN1;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    proc format cntlin=work.cntlin2;
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to access table: CNTLIN2;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    proc format cntlin=work.cntlin3;
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to access table: CNTLIN3;
        %put_aml_error;
        %goto MACRO_END;
    %end;

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*  BUILD FORMATS                                                             */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* -------------------------------------------------------------------------- */
/* Get active alerts and create date                                          */
/* -------------------------------------------------------------------------- */
    data work.alert_age ;
        set seg_kc.fsk_alert;
        keep age_group ;
        if alert_status_code = 'ACT' ;
        age_group = put(&max_create - datepart(create_date),agerange.) ;
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to create table: ALERT_AGE;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    PROC FREQ data=work.alert_age  noprint ;
        tables age_group /
            nocum
            nopercent
            alpha=0.0500
            scores=table
            out=work.frequency ;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to create table: FREQUENCY;
        %put_aml_error;
        %goto MACRO_END;
    %end;

/* -------------------------------------------------------------------------- */
/* Get Number for each range and sort by number                               */
/* -------------------------------------------------------------------------- */
    data work.assign_num ;
        set work.frequency;
        drop percent;
        num =  input(put(input(scan(age_group,1),8.),rangenum.),8.);

    PROC SORT data=assign_num out=sorted_assignment;
        by num;
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to access table: FREQUENCY;
        %put_aml_error;
        %goto MACRO_END;
    %end;

/* -------------------------------------------------------------------------- */
/*  Fill in empty ranges - with zeros for all ranges                          */
/*  create tabulate link variable                                             */
/* -------------------------------------------------------------------------- */
/* ------------------------------------------------------------------- */
/*    Generate Macros                                                  */
/* ------------------------------------------------------------------- */
    %label_maker(basefile=dist_active_alerts_meta
            ,table_name=DIST_ACTIVE_CHART_MACROS_META
            ,generate=MACVAR) ;



    data work.age_groups;
        merge work.sorted_assignment work.template  ;
        by num;
        format num numrange.  tabulate_link chart_link $120.;
        label num="&age_in_days"
                tabulate_link="&age_in_days"
                count="&alerts";
        if age_group = ' ' then age_group = label;
        if count = .  then count = 0;
        start = input(scan(label,1),8.);
        end = input(scan(label,2),8.);

        tabulate_link ='<a  HREF="alert_age.htm#' || trim(left(label)) || '"> ' ||
            trim(left(label))  || '</a> ';
        chart_link ='HREF="alert_age.htm#' || trim(left(label)) || '"' ;

        if count = 0 then do;
            tabulate_link ='<a  HREF="no_rec.htm"> ' || trim(left(label)) ||' </a> ';
            chart_link ='HREF="no_rec.htm" '  ;
        end;
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to create table: AGE_GROUPS;
        %put_aml_error;
        %goto MACRO_END;
    %end;

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*  CREATE BAR CHART AND TABLE                                                */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* -------------------------------------------------------------------------- */
/*  Setup the output environment                                              */
/* -------------------------------------------------------------------------- */
    /* If output directory exists --- remove old reports        */
    /* use lower case 'deldir' to indicate that you want to     */
    /* remove directory - macro converts to proper code         */
    /* depending on platform  - unix or windows                 */
    %if %sysfunc(fileexist(&outputDirectory)) %then %do;
        %aml_execute_command(deldir  &outputDirectory);
    %end;
    /* Create output directory                                  */
    %aml_execute_command(mkdir &outputDirectory);

    OPTIONS   device=&_outputdevice  ;
    GOPTIONS  xpixels=480  ypixels=360;

    ODS LISTING CLOSE;
    ODS HTML PATH="&outputDirectory"     (URL=NONE)
            BODY="&outputBody"
            NOGTITLE NOGFOOTNOTE
            STYLE=&_outputstyle   ;
    PATTERN1 COLOR=&_bar_color;
    AXIS1 label=("&alerts") MINOR=NONE;
    AXIS2 label=("&age_in_days") MINOR=NONE;
    &_title;
    title2 "&title2";
    footnote1  "&footnote";

    /*  FOOTNOTE8 height=&_fheight justify=left "&aml_footnote_date" justify=right "&aml_footnote_version";*/

    PROC gchart DATA=work.age_groups ;
        &_chart_type num /
        SUMVAR=COUNT
        RAXIS=AXIS1
        MAXIS=AXIS2
        AUTOREF          html=chart_link html_legend=chart_link
        DISCRETE
        SHAPE=&_bar_shape
        TYPE=sum
        COUTLINE=BLACK
        FRAME  ;
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to access table: AGE_GROUPS;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    TITLE;FOOTNOTE;

/* -------------------------------------------------------- */
/* Output table of Values                                   */
/* -------------------------------------------------------- */
/*Do we want a table report  - default is Y  */
    %if %upcase(&_table_report_YN) = Y %then %do;

        &_title;
        title2 "&title2";
        footnote1  height=&_fheight "&footnote";

        /*  FOOTNOTE8 height=&_fheight justify=left "&aml_footnote_date" justify=right "&aml_footnote_version";*/
        PROC TABULATE DATA=work.age_groups
            FORMAT=COMMA6.;
            VAR COUNT;
            CLASS tabulate_link / MISSING ORDER=DATA  ;
            TABLE tabulate_link,
            COUNT*
            Sum={LABEL=' '};
        RUN;
        %aml_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _amlmsg_= NOTE: AML: Unable to create table: AGE_GROUPS;
            %put_aml_error;
            %goto MACRO_END;
        %end;
        TITLE;FOOTNOTE;
    %END;

ODS _ALL_ CLOSE;
ODS LISTING;
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*  GET ACTIVE ALERT RECORDS                                                  */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    PROC SQL noprint;
        CREATE TABLE work.Active_alerts AS
        SELECT
            a.ALERT_ID                        ,
            a.PRODUCT_TYPE                    ,
            a.ALERT_STATUS_CODE               ,
            a.MONEY_LAUNDERING_RISK_SCORE     ,
            a.TERROR_FINANCING_RISK_SCORE     ,
            a.ALERT_DESCRIPTION               ,
            a.ALERT_SUPPRESSED_IND            ,
            a.PRIMARY_ENTITY_LEVEL_CODE       ,
            a.PRIMARY_ENTITY_NUMBER           ,
            a.PRIMARY_ENTITY_NAME             ,
            a.SCENARIO_ID                     ,
            a.SCENARIO_NAME                   ,
            a.SUPPRESSION_END_DATE            FORMAT=nldatm.,
            a.ACTUAL_VALUES_TEXT              ,
            a.CREATE_DATE                     FORMAT=nldatm.,
            a.CREATE_USER_ID
        FROM seg_kc.FSK_ALERT AS a
        WHERE a.ALERT_STATUS_CODE = 'ACT'
        ORDER BY a.CREATE_DATE DESCENDING  ;
    QUIT;
    %aml_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to create table: ACTIVE_ALERTS;
        %put_aml_error;
        %goto MACRO_END;
    %end;

    data work.active_alerts_age;
        set work.active_alerts;
        AGE_IN_DAYS = &max_create - datepart(create_date);
        if  CREATE_USER_ID      = ' ' then CREATE_USER_ID    = 'NO CREATE ID';

/* ------------------------------------------------------------------- */
/*    Generate Labels                                                  */
/* ------------------------------------------------------------------- */
    %label_maker(basefile=dist_active_alerts_meta
            ,table_name=DIST_ACTIVE_LABELS_META
            ,generate=LABEL) ;

        FORMAT PRIMARY_ENTITY_LEVEL_CODE    $subject.;

        %let varlist1 =  %sysfunc(compbl(
            VAR AGE_IN_DAYS
            ALERT_ID
            ALERT_DESCRIPTION
            PRIMARY_ENTITY_LEVEL_CODE
            PRIMARY_ENTITY_NUMBER
            PRIMARY_ENTITY_NAME
            ));
        %let varlist2 =  %sysfunc(compbl(
            CREATE_DATE
            MONEY_LAUNDERING_RISK_SCORE
            TERROR_FINANCING_RISK_SCORE
            SCENARIO_NAME
            ACTUAL_VALUES_TEXT
            ));
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to access table: ACTIVE_ALERTS;
        %put_aml_error;
        %goto MACRO_END;
    %end;

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*  GENERATE CODE FOR ODS HTML LISTING BY DAYS OLD GROUPING                   */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
* -------------------------------------------------------------;
* Allocate catalog for source
* -------------------------------------------------------------;
proc datasets lib=work nolist mt=cat;
   delete codefile;
quit;
filename tempcode catalog "work.codefile.tempcode.source";
/* ------------------------------------------------------- */
/* This code generates the code necessary to create a      */
/* listing by group and owners of alerts                   */
/*                                                         */
/* ------------------------------------------------------- */
/* ------------------------------------------------------------------- */
/*   Generate Macros                                                   */
/* ------------------------------------------------------------------- */
    %label_maker(basefile=dist_active_alerts_meta
            ,table_name=DIST_ACTIVE_RPT_MACROS_META
            ,generate=MACVAR) ;

    data _null_;
        set  work.age_groups  end=last;
        format line $4096.  verbage $17.;
        file tempcode lrecl=4096;
        IF _n_ = 1 then do;

            put "OPTIONS   DEV=&_outputdevice ;" ;
            put " ";
            put "ODS LISTING CLOSE; ";
            put "ODS HTML PATH='&outputDirectory'       (URL=NONE) ";
            put "         BODY='alert_age.htm'                      ";
            put "           NOGTITLE NOGFOOTNOTE                   ";
            put "            STYLE=&_outputStyle._prt  ;                ";
            put " ";
        end;
        if count = 0 then return;

        line = 'ods html anchor="' ||  trim(label)  ||  '";'  ;
        put  line;
        verbage = " &verbage ";
        if count = 1 then verbage = " &verbage1 ";
        line = 'Title1 "' || strip(put(count,8.)) || ' ' || strip(verbage) || ' ' ||
                             strip(put(start,8.)) || " &title_to " ||
                             strip(put(end,8.))   || ' &title_days_old " ;';
        put  line;
        put 'proc print data=work.active_alerts_age  noobs label ;'  ;
        line="&varlist1"  ;
        put  line;
        line="&varlist2" || ';' ;
        put  line;
        line = 'where ' || trim(left(put(start,8.))) || ' le age_in_days le ' ||
                           trim(left(put(end,8.))) || ' ;';
        put line;
        put 'run;';
        put " ";
        if last then do;
            put 'ODS _ALL_ CLOSE; ';
            put 'ODS LISTING;   ';
        end;
    run;
    %aml_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _amlmsg_= NOTE: AML: Unable to access table: AGE_GROUPS;
        %put_aml_error;
        %goto MACRO_END;
    %end;
    /* ---------------------------------------------------------- */
    /*  INCLUDE GENERATED CODE                                    */
    /* ---------------------------------------------------------- */
    %include tempcode;
    run;

    /* ---------------------------------------------------------- */
    /*  GENERATE NO RECORD FOUND HTML PAGE                        */
    /* ---------------------------------------------------------- */
    %no_record_htm(Msg1 = %superq(no_rec_msg),
        msg2=,
        msg3=   );
    /* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv */
    /* ---------------------------------------------------------- */
    /*            END OF MAIN PROCESSING MACRO                    */
    /*             DIST_ACTIVE_ALERTS_REPORT                      */
    /* ---------------------------------------------------------- */
    /* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv */
%MACRO_END:   /*  EXIT MACRO */

    /* Delete report macro variables so you don't get macro variable collisions   */
    %symdel title
        chart_type      _chart_type
        bar_shape       _bar_shape
        bar_color       _bar_color
        num_groups      _num_groups
        table_report_YN _table_report_YN / nowarn ;

    /* Delete title macro variable */
    %label_maker(basefile=dist_active_alerts_meta
        ,table_name=REPORT_TITLE_MACRO_META
        ,generate=MACDEL) ;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %execute_macro(amlerror);
    %else %do;
        %execute_macro(post&_amlmodule_);
        %execute_macro(amlsuccess);
    %end;

%mend dist_active_alerts_report;
