/******************************************************************************/
/*                  S A S   S A M P L E   L I B R A R Y                       */
/*                                                                            */
/*    NAME: lov_report.sas                                                    */
/*   TITLE: List of Values (FSK_LOV) Report                                   */
/* PRODUCT: SAS Anti-Money Laundering                                         */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/******************************************************************************/
/* DESCRIPTION: Create report of fsk_lov table
/* COMMENTS:    Required macros variable settings:
/*                outputDirectory - Directory of where the output HTML/EXCEL
/*                   report is to be placed
/*                outputBody - name of the member in output directory -
/*                   typically the value is: body
/*                outputDevice - the output graphic format that will be
/*                   generated typically the value is: JAVAIMG
/*             
/*              External macros used:
/*                 %put_aml_error
/*                 %aml_rcset
/*                 %aml_execute_command
/*                 %label_maker
/*             
/*              Valid values for report_type:  HTM, EXCEL
/*
/******************************************************************************/

/******************************************************************************/
/*  Copyright (c) 2012 by SAS Institute Inc., Cary, NC 27513 USA              */
/*  ---All Rights Reserved.                                                   */
/******************************************************************************/

%macro lov_report(report_type=HTM) ;

/* Set Module */
     %local _AMLMODULE_;
     %let _AMLMODULE_ = LOV_REPORT;

/* Set error code to 0 and message to blank*/
     %let trans_rc = 0;
     %let _amlmsg_=;

    /* Call pre stub macro */
     %execute_macro(pre&_amlmodule_);

     %let _report_type=HTM;         /* OPTIONAL MACRO VARIABLE - set default */
     %if %SYMEXIST(report_type) = 1 %then %do ;   /* Does variable exist?    */
        %if &report_type ne   %then %do ;         /* Does it have a value?   */
           %if ( %upcase(&report_type) = HTM or
              %upcase(&report_type) = EXCEL )  %then %do ;
                 %let _report_type =  &report_type  ;
           %end;
        %end;
     %end;

/*==============================================================================
   SELECT DATA SECTION
==============================================================================*/
   %aml_get_observation_count(table=seg_kc.fsk_lov);
   %if &aml_obs_count eq -1 %then %do;
      %let _amlmsg_= NOTE: AML: seg_kc.fsk_lov does not exist or is not accessible.;
      %put_aml_error;
      %goto MACRO_END;
   %end;
   %if &aml_obs_count eq 0 %then %do;
      %let _amlmsg_= NOTE: AML: seg_kc.fsk_lov has 0 records.  &_AMLMODULE_ is being skipped.;
      %put_aml_error;run;
      %goto MACRO_END;
   %end;

/*---------------------------------------------------------
   Sort LOV table
---------------------------------------------------------*/
      proc sort data= seg_kc.fsk_lov
         out=work.sorted_lov (keep = lov_type_name lov_type_code lov_type_desc);
         by lov_type_name lov_sort_position_number ;

      /* define labels for report colums */
      %label_maker(basefile=lov_table_report_meta ,
         table_name=lov_table_labels_meta ,
         generate=label ) ;
      run ;
      /* Check for errors */
      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do ;
         %let _amlmsg_=NOTE: AML: Unable to create table: work.sorted_lov. &_AMLMODULE_ is being skipped.;
         %put_aml_error ;
         %goto MACRO_END ;
      %end ;

/*==============================================================================
   REPORT SECTION
==============================================================================*/
/* ---------------------------------------------------------*/
/* Setup the output environment
/* ---------------------------------------------------------*/
/* Delete output directory if it exists
/* use lower case 'deldir' to indicate that you want to
/* remove directory - macro converts to proper code
/* depending on platform  - unix or windows
/* ---------------------------------------------------------*/
      %if %sysfunc(fileexist(&outputDirectory)) %then %do;
         %aml_execute_command(deldir  &outputDirectory);
      %end;
      /* Create output directory                                  */
      %aml_execute_command(mkdir &outputDirectory);

      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_= NOTE: AML: Unable to create directory: &outputDirectory;
         %put_aml_error;
         %goto MACRO_END;
      %end;

      /* Get title from xml file*/
      %label_maker(basefile=lov_table_report_meta ,
         table_name=lov_table_macros_meta ,
         generate=macvar ) ;

      options   device=&outputdevice  ;

      %if &_report_type=HTM %then %do;
            goptions  xpixels=480  ypixels=360;
            /* Set ODS parms                                            */
            ods listing close;
            ods html path="&outputdirectory"     (url=none)
               BODY="&outputBody"
               NOGTITLE NOGFOOTNOTE
               STYLE=&_outputstyle   ;

            title1 "&title1";
            footnote1 height=3 "&footnote1";
            footnote2 height=3 "&footnote2";
         /* Generate report */
            proc report data = work.sorted_lov   nowindows
               contents='LOV Table Report' ;
            column
               lov_type_name
               lov_type_code
               lov_type_desc  ;
               define lov_type_name / order ;
            run ;
      %end;
      %else %do;
            proc export data= work.sorted_lov
                        outfile= "&outputdirectory./body.xls"
                        dbms=excel replace;
                 sheet="&title1";
            run;
      %end;
      /* Check for errors */
      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_ = NOTE: AML: Unable to create report using work.sorted_lov;
         %put_aml_error;
         %goto MACRO_END;
      %end;

      title ;
      /* Reset ODS parms */
      ods _all_ close;
      ods listing;

      /* delete macro variables */
      %label_maker(basefile=lov_table_report_meta ,
         table_name=lov_table_macros_meta ,
         generate=macdel ) ;

      /* delete work dataset */
      proc datasets nowarn nolist lib=work ;
         delete
            sorted_lov ;
      quit ;

%MACRO_END:   /*  EXIT MACRO */

      %if &trans_rc eq 3 or &trans_rc ge 5 %then
         %execute_macro(amlerror);
      %else %do;
         %execute_macro(post&_amlmodule_);
         %execute_macro(amlsuccess);
      %end;


%mend lov_report ;
