/******************************************************************************/
/*                  S A S   S A M P L E   L I B R A R Y                       */
/*                                                                            */
/*    NAME: monthly_trend_report.sas                                          */
/*   TITLE: Monthly Trend Report                                              */
/* PRODUCT: SAS Anti-Money Laundering                                         */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/******************************************************************************/
/* Description: Produces a monthly bar chart of the number of alert items
/*              created each month.  Also a optional tabular report. Default
/*              is to produce tabular report. See optional parms below to
/*              turn it off.
/*
/* PARAMETERS:
/* <!-- ODS Parms -->
/*        outputDirectory = DIRECTORY FOR THIS REPORT
/*        outputBody = REPORT HTML BODY NAME:  body.htm
/*        outputDevice = GRAPHICS RENDERING:  javaimg/actximg
/*      _outputStyle = STYLE TO APPLY:  sasweb, banker etc
/* <!-- Report Parms -->
/*        num_months = NUMBER OF MONTHS TO TREND: 13
/*        title = REPORT TITLE
/*        bar_color = BAR_COLOR:  blue
/*        bar_shape = SHAPE OF BARS:  block
/*        table_report_YN>N = PRODUCE A AN APPENDED TABULATE REPORT:  Y
/*        curr_prev = include current or prev month
/*        display_x_axis_label_YN display x-axis label: Y
/*        display_Y_axis_label_YN display y-axis label: Y
/* <!-- Called routines and macros -->
/*      %aml_rcset
/*      %put_aml_error
/*      %aml_execute_command
/*
/******************************************************************************/

/******************************************************************************/
/*  Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA         */
/*  ---All Rights Reserved.                                                   */
/******************************************************************************/

%macro monthly_trend_report ;

   /* Set Module Name */
    %local _AMLMODULE_;
    %let _AMLMODULE_=MONTHLY_TREND_REPORT;

   /* Call preMONTHLY_TREND_REPORT stub macro */
    %execute_macro(pre&_amlmodule_);

   /* Set error code to 0 and message to blank*/
    %let trans_rc = 0;
    %let _amlmsg_=;

   /*--------------------------------------------------------
    * Make sure the AML start datetime has been set.
    * Assign its value to macro variable aml_start_datetime.
    *--------------------------------------------------------*/
    %aml_start_datetime_read;
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %goto MACRO_END2;
    %end;

/*--------------------------------------------------------------------
    Generate Macro Variables
---------------------------------------------------------------------*/
    /* Get title from xml file*/
    %label_maker(basefile=monthly_trend_meta
            ,table_name=REPORT_TITLE_MACRO_META
            ,generate=MACVAR) ;

    %label_maker(basefile=monthly_trend_meta
            ,table_name=ALERTS_BY_MONTH_META
            ,generate=MACVAR) ;

   DATA _NULL_;

   %let _outputdevice =javaimg;
   %IF %SYMEXIST(outputdevice) = 1  %then %do ;  /* Does variable exist?    */
       %IF &outputdevice ne %then %do ;          /* Does it have a value?   */
           %let _outputdevice = &outputdevice;   /* Then set value          */
       %end;
   %end;

   %let _num_months=13;                         /* OPTIONAL MACRO VARIABLE - set default */
   %IF %SYMEXIST(num_months) = 1 %then %do ;    /* Does variable exist?    */
       %IF &num_months ne   %then %do ;         /* Does it have a value?   */
           %let _num_months=&num_months;               /* Then set value          */
       %end;
   %end;

   %let _bar_color=BLUE;                       /* OPTIONAL MACRO VARIABLE - set default */
   %IF %SYMEXIST(bar_color) = 1 %then %do ;    /* Does variable exist?    */
       %IF &bar_color ne   %then %do ;         /* Does it have a value?   */
           %let _bar_color=&bar_color;         /* Then set value          */
       %end;
   %end;

   %let _bar_shape = block;                   /* OPTIONAL MACRO VARIABLE - set default */
   %IF %SYMEXIST(bar_shape) = 1  %then %do ;  /* Does variable exist?    */
       %IF &bar_shape ne %then %do ;          /* Does it have a value?   */
           %let _bar_shape =  &bar_shape;     /* Then set value          */
       %end;
   %end;

   %let _table_report_YN = Y;                        /* OPTIONAL MACRO VARIABLE - set default */
   %IF %SYMEXIST(table_report_YN) = 1  %then %do ;   /* Does variable exist?    */
       %IF &table_report_YN ne %then %do ;           /* Does it have a value?   */
           %if ( %upcase(&table_report_YN) = Y or
           %upcase(&table_report_YN) = N )  %then %do ;
               %let _table_report_YN = &table_report_YN;   /* Then set value          */
           %end;
       %end;
   %end;

   %let _curr_prev = CURR  ;
   %IF %SYMEXIST(curr_prev) = 1  %then %do ;   /* Does variable exist?    */
       %IF &curr_prev ne %then %do ;           /* Does it have a value?   */
           %if  %upcase(&curr_prev) =  PREV %then %let _curr_prev = PREV;
       %end;
   %end;

   %let _display_x_axis_label_YN = Y ;                    /* OPTIONAL MACRO VARIABLE - set default */
   %if %symexist(display_x_axis_label_YN) = 1 %then %do ;  /* Does variable exist?    */
      %if &display_x_axis_label_YN ne %then %do ;          /* Does it have a value?   */
         %if ( %upcase(&display_x_axis_label_YN) = Y  
         or %upcase(&display_x_axis_label_YN) = N )  %then %do ;
            %let _display_x_axis_label_YN = &display_x_axis_label_YN ;     /* Then set value          */
         %end ;
      %end ;
   %end ;

   %let _display_y_axis_label_YN = Y ;                    /* OPTIONAL MACRO VARIABLE - set default */
   %if %symexist(display_y_axis_label_YN) = 1  %then %do ;  /* Does variable exist?    */
      %if &display_y_axis_label_YN ne %then %do ;          /* Does it have a value?   */
         %if ( %upcase(&display_y_axis_label_YN) = Y 
         or %upcase(&display_y_axis_label_YN) = N )  %then %do ;
            %let _display_y_axis_label_YN = %upcase(&display_y_axis_label_YN) ;     /* Then set value          */
         %end ;
      %end ;
   %end ;

   /*----------------------------------------------------------*/
   /* SET MACRO VARIABLES FOR MONTH RANGES AND TITLES
   /*----------------------------------------------------------*/

   data _null_;
   todays_date = datepart(&aml_start_datetime);
   number_months = input("&_num_months",8.);
   if "&_curr_prev" = "PREV" then do;
        intnx_ind= -1;                      /* set to previous month  */
   end;
   if "&_curr_prev" = "CURR" then do;
        intnx_ind= 0;                       /* unless they wnat to do current month to date */
        number_months = number_months -1;   /* Adjust look back months for curr month*/
   end;

    call symput('Begin_year_month',put(intnx('month',todays_date,-number_months,'B'),yymmd7.));  /* Get first day of range     */
    call symput('End_year_month',put(intnx('month',todays_date,intnx_ind,'B'),yymmd7.));              /* Get very last day of range */


    call symput('Begin_range',put(intnx('month',todays_date,-number_months,'B'),8.));  /* Get first day of range     */
    call symput('End_range',put(intnx('month',todays_date,intnx_ind,'E'),8.));              /* Get very last day of range */


   /*----------------------------------------------------------*/
   /* CREATE A TEMPLATE FILE OF ALL MONTHS IN RANGE
   /*----------------------------------------------------------*/
      data zero_template;
         keep year_month ;
         format year_month yymmd7.;
         do i = 0 to &_num_months - 1;
            year_month = intnx('month',&begin_range,i);
            output zero_template;
         end;
      run;

      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_= NOTE: AML: Unable to create table: ZERO_TEMPLATE;
         %put_aml_error;
         %goto MACRO_END;
      %end;

   /*----------------------------------------------------------*/
   /* ASSIGN CREATE_DATES TO MONTH BUCKETS AND SUMMARIZE
   /*----------------------------------------------------------*/
      %if %sysfunc(exist(seg_kc.fsk_alert)) %then %do;
         data work.month_buckets;
            set seg_kc.fsk_alert;
            keep year_month count;
            format  month_created yymmd7. ;
            year_month = intnx('month',datepart(create_date),0,'b');
            count=1;
            if  &Begin_range <= year_month <= &End_range  then output  work.month_buckets;
         run;
         %aml_rcset(&syserr);
         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _amlmsg_= NOTE: AML: Unable to create table: MONTH_BUCKETS;
            %put_aml_error;
            %goto MACRO_END;
         %end;
      %end;
      %else %do;
         %let _amlmsg_= NOTE: AML: SEG_KC.FSK_ALERT does not exist or is not accessible;
         %put_aml_error;
         %goto MACRO_END;
      %end;
      %let dsid=%sysfunc(open(work.month_buckets,i));
      %let num_obs=%sysfunc(attrn(&dsid,nobs));
      %let rc=%sysfunc(close(&dsid));
      %if &num_obs = 0 %then %do;
         %let _amlmsg_= NOTE: AML: Zero records selected for this report, no report will be produced;
         %put_aml_error;
         %goto MACRO_END;
      %end;

      proc means  data=work.month_buckets
          noprint   chartype maxdec=0  vardef=df
          nway sum nonobs  ;
          var count;
          class year_month /  order=unformatted ascending;
      output  out=work.summarized
              sum()=
          /   ways inherit   ;
      run;
      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_= NOTE: AML: Unable to create table: SUMMARIZED;
         %put_aml_error;
         %goto MACRO_END;
      %end;


   /*----------------------------------------------------------*/
   /* FILL IN MISSING MONTHS WITH ZEROS FROM TEMPLATE
   /*----------------------------------------------------------*/
      data work.chart_data;
      merge  work.summarized   work.zero_template   ;
          by year_month;
          label year_month ='Month'    count='Count' ;
          drop _way_ _type_ _freq_;
      if count = . then count= 0;
         %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_= NOTE: AML: Unable to create table: CHART_DATA;
         %put_aml_error;
         %goto MACRO_END;
      %end;

      proc means  data=work.chart_data
          noprint   chartype maxdec=1  vardef=df
          mean  min  max sum nonobs  ;
      var count;
      output  out=work.total_avg_min_max
              mean()=average  min()=minimum max()=maximum  sum()=total
          /  inherit  ;
      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_= NOTE: AML: Unable to create table: TOTAL_AVG_MIN_MAX;
         %put_aml_error;
         %goto MACRO_END;
      %end;
      run;


      data _null_;
      set work.total_avg_min_max;
      call symput('average',put(average,8.1));
      call symput('minimum',put(minimum,8.));
      call symput('maximum',put(maximum,8.));

      /* S0549570 (hk1012279) */
      call symput('total_alerts',put(total,8.));

      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_= NOTE: AML: Unable to read table: TOTAL_AVG_MIN_MAX;
         %put_aml_error;
         %goto MACRO_END;
      %end;
      run;

/*--------------------------------------------------------------------
    Generate Title Macro Variables
---------------------------------------------------------------------*/
%label_maker(basefile=monthly_trend_meta
            ,table_name=ALERTS_BY_MONTH_TITLE_META
            ,generate=MACVAR) ;

   /*----------------------------------------------------------*/
   /* Setup the output environment
   /*----------------------------------------------------------*/
   /* Delete output directory if it exists
   /* use lower case 'deldir' to indicate that you want to
   /* remove directory - macro converts to proper code
   /* depending on platform  - unix or windows
   /*----------------------------------------------------------*/
      %if %sysfunc(fileexist(&outputDirectory)) %then %do;
         %aml_execute_command(deldir  &outputDirectory);
      %end;
      /* Create output directory                                  */
      %aml_execute_command(mkdir &outputDirectory);
      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_= NOTE: AML: Unable to create directory: &outputDirectory;
         %put_aml_error;
         %goto MACRO_END;
      %end;
   OPTIONS   device=&_outputdevice  ;
   GOPTIONS  xpixels=480  ypixels=360;

   /* Set ODS parms                                            */
   ODS LISTING CLOSE;
   ODS HTML PATH="&outputDirectory"     (URL=NONE)
           BODY="&outputBody"
           NOGTITLE NOGFOOTNOTE
           STYLE=&_outputstyle   ;


   /*----------------------------------------------------------*/
   /* Do BAR Chart
   /*----------------------------------------------------------*/
   PATTERN1 COLOR=&_bar_color;
   TITLE;
   TITLE1 "&title";
   TITLE2  "&title2";
   FOOTNOTE;
FOOTNOTE1 height=&_fheight  "&footnote1";
FOOTNOTE2 height=&_fheight  "&footnote2";
    /*  FOOTNOTE8 height=&_fheight justify=left "&aml_footnote_date" justify=right "&aml_footnote_version";*/

   %if &_display_x_axis_label_YN eq N  %then %do ;   
      %let xaxis_label = ;
   %end;
   %if &_display_y_axis_label_YN eq N  %then %do ;   
      %let yaxis_label = ;
   %end;

   Axis1
       COLOR=BLACK
       STYLE=1
       LABEL=(HEIGHT=12pt JUSTIFY=Right "&yaxis_label");
   Axis2
       COLOR=BLACK
       STYLE=1
       VALUE=(HEIGHT=12pt  color=black angle=-90 )

       LABEL=(HEIGHT=12pt JUSTIFY=Center "&xaxis_label");
   PROC GCHART DATA=WORK.CHART_DATA;
   VBAR3D   year_month
    /
       SUMVAR=Count
       RAXIS=AXIS1
       MAXIS=AXIS2

       SHAPE=&_bar_shape
       FRAME
       DISCRETE
       TYPE=SUM
       NOLEGEND
       CREF=BLACK
       AUTOREF ;
   RUN;
      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_= NOTE: AML: Unable to access table: CHART_DATA;
         %put_aml_error;
         %goto MACRO_END;
      %end;

   TITLE; FOOTNOTE;
   /*----------------------------------------------------------*/
   /* Output table of Values
   /*----------------------------------------------------------*/
   /*Do we want a table report  - default is Y  */
   %if %upcase(&_table_report_YN) = Y %then %do;
       PROC TABULATE DATA=WORK.CHART_DATA
           STYLE={FONT_SIZE=2 FONT_WEIGHT=MEDIUM JUST=CENTER VJUST=MIDDLE} ;
           VAR count;
           CLASS year_month / MISSING ORDER=UNFORMATTED;
           TABLE /* Row Dimension */
               year_month={LABEL=''}
               ALL={LABEL="&total" },
               /* Column Dimension */
               count={LABEL="&number_of_alerts" STYLE={NOBREAKSPACE=ON}}*(
                 Sum={LABEL="&alerts" STYLE={NOBREAKSPACE=ON}}*F=&format_nlnum9
                 PctSum={LABEL="&percent_of_total" STYLE={NOBREAKSPACE=ON}}*F=&format_nlnum51)
               /* Table Options */
               / BOX={LABEL="&month" STYLE={NOBREAKSPACE=ON}};

        TITLE1 "&title";
        TITLE2  "&title2";
        FOOTNOTE1 height=&_fheight  "&footnote1";
        FOOTNOTE2 height=&_fheight  "&footnote2";
    /*  FOOTNOTE8 height=&_fheight justify=left "&aml_footnote_date" justify=right "&aml_footnote_version";*/
       RUN;
      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_= NOTE: AML: Unable to access table: CHART_DATA;
         %put_aml_error;
         %goto MACRO_END;
      %end;
       TITLE;FOOTNOTE;
   %end;
   /* End of task code. */
   ODS _ALL_ CLOSE;
   ODS LISTING;
      %MACRO_END:   /*  EXIT MACRO  */

         /* Delete report macro variables so you don't get macro variable collisions   */
              %symdel
                        curr_prev    _curr_prev
                        bar_color    _bar_color
                        bar_shape    _bar_shape
                        num_months   _num_months
                        table_report_YN  _table_report_YN / nowarn ;
         /*--------------------------------------------------------------------
             SYMDEL Macro Variables
         ---------------------------------------------------------------------*/
             %label_maker(basefile=monthly_trend_meta
                     ,table_name=ALERTS_BY_MONTH_META
                     ,generate=MACDEL) ;
         /*--------------------------------------------------------------------
             SYMDEL Title Macro Variables
         ---------------------------------------------------------------------*/
             %label_maker(basefile=monthly_trend_meta
                     ,table_name=ALERTS_BY_MONTH_TITLE_META
                     ,generate=MACDEL) ;
            /* Delete tirle macro variable */
            %label_maker(basefile=monthly_trend_meta
                    ,table_name=REPORT_TITLE_MACRO_META
                    ,generate=MACDEL) ;


      %if &trans_rc eq 3 or &trans_rc ge 5 %then
         %execute_macro(amlerror);
      %else %do;
         %execute_macro(post&_amlmodule_);
         %execute_macro(amlsuccess);
      %end;

%mend monthly_trend_report;
