/******************************************************************************/
/*                  S A S   S A M P L E   L I B R A R Y                       */
/*                                                                            */
/*    NAME: scenario_audit_report.sas                                         */
/*   TITLE: Scenario Audit Report                                             */
/* PRODUCT: SAS Anti-Money Laundering                                         */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/******************************************************************************/
/* Description: Produces a listing of all active headers.
/*              Executed as needed or on a regular basis.
/*
/* OPERATIONAL PARMS
/* REQUIRED
/* <!-- ODS Parms -->
/*        outputDirectory = DIRECTORY FOR THIS REPORT
/*        outputBody = REPORT HTML BODY NAME:  body.htm
/*        outputDevice = GRAPHICS RENDERING:  javaimg/actximg
/*        _outputStyle = STYLE TO APPLY:  sasweb, banker etc
/* <!-- Report Parms -->
/*        title = REPORT TITLE
/*        number_days = number days to go back for changes --- default = 999
/* <!-- Called routines and macros -->
/*      %aml_rcset
/*      %put_aml_error
/*      %aml_execute_command
/*      %label_maker
/*      %no_record_htm
/*
/******************************************************************************/

/* ************************************************************************** */
/*  Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA         */
/*  ---All Rights Reserved.                                                   */
/* ************************************************************************** */

%macro scenario_audit_report;

   /* Set Module Name */
    %local _AMLMODULE_;
    %let _AMLMODULE_=SCENARIO_AUDIT_REPORT ;

   /* Call preSCENARIO_AUDIT_REPORT stub macro */
    %execute_macro(pre&_amlmodule_);

   /* Set error code to 0 and message to blank*/
    %let trans_rc = 0;
    %let _amlmsg_=;

   /*--------------------------------------------------------
    * Make sure the AML start datetime has been set.
    * Assign its value to macro variable aml_start_datetime.
    *--------------------------------------------------------*/
    %aml_start_datetime_read;
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %goto MACRO_END2;
    %end;

    %global _number_days ;
    %let _number_days = 999;
   
    /*--------------------------------------------------------------------
    Generate MACRO variables for report
    ---------------------------------------------------------------------*/
    %label_maker(basefile=scenario_audit_meta
        ,table_name=SCENARIO_AUDIT_RPT_MACROS_META
        ,generate=MACVAR) ;

    /* Get title from xml file*/
    %label_maker(basefile=scenario_audit_meta
        ,table_name=REPORT_TITLE_MACRO_META
        ,generate=MACVAR) ;


  %macro aml_init_and_verify_params;
    /* -------------------------------------------------------------------- */
    /*   SET INITIAL VALUES
    /* -------------------------------------------------------------------- */
    DATA _NULL_;
     /* Verify and set all report parameters  */
     /* ------------------------------------------------------------------
        NUMBER DAYS TO LOOK BACK
        ------------------------------------------------------------------ */
     %IF %SYMEXIST(number_days) = 1  %then %do;  /* Does variable exist?   */
         %IF &number_days ne %then %do ;         /* Does it have a value?  */
             %let _number_days = &number_days;   /* Then set value         */
         %end;
     %end;
     %global date_limit ;
     format date_limit   date9.;
     date_limit = datepart(&aml_start_datetime) - &_number_days;
     call symput("date_limit",strip(put(date_limit,8.)));
    run;
  %mend aml_init_and_verify_params;
   
  %macro aml_select_updated_scenarios;
   /* -------------------------------------------------------------------
      CREATE DUMMY FILE FOR APPENDING
      ------------------------------------------------------------------- */
   data work.changed_records;
       attrib
           SCENARIO_NAME           format=$35.
           PARAMETER_TYPE          format=$20.
           PARAMETER               format=$35.
           ATTRIBUTE               format=$35.
           ORIGINAL_VALUE          format=$1024.
           UPDATED_VALUE           format=$1024.
           USER                    format=$35.
           CHANGE_DATE             format=DATETIME20.0
           UPDATED_VERSION         format=6.0
           SORT_KEY                format=8.
           ;
           SCENARIO_NAME   = ' ';
           PARAMETER_TYPE  = ' ';
           PARAMETER  = ' ';
           ORIGINAL_VALUE   = ' ';
           UPDATED_VALUE  = ' ';
           USER   = ' ';
           CHANGE_DATE    = 0  ;
           UPDATED_VERSION =  0 ;
           SORT_KEY     = -1;
           RUN;

   /* -------------------------------------------------------------------
      GET UPDATED SCENARIOS
      ------------------------------------------------------------------- */
   proc sql;
   create table work.updated_scenarios as
   select a.*
   from seg_kc.fsk_scenario as a    
   where ( a.scenario_id not  = -1 and a.version_number > 1  )
   order by a.scenario_name, a.version_number;
   quit;

   %if &sqlobs = 0 %then %do;
   /* ------------------------------------------------------------------------------
      NO RECORDS TO PRINT --- Put warning message and put html page with message. 
   --------------------------------------------------------------------------------- */
         %let _amlmsg_=NOTE: AML: Zero records selected for this report, no report will be produced;
         %put_aml_error;
         /* If output directory exists --- remove old reports      */
         %if %sysfunc(fileexist(&outputDirectory)) %then %do;
            %aml_execute_command(deldir  &outputDirectory);
         %end;
         /* Create new output directory                            */
         %aml_execute_command(mkdir &outputDirectory);
         %aml_rcset(&syserr);
         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _amlmsg_=ERROR: AML: Unable to create directory: &outputDirectory;
            %put_aml_error;
         %end;
         /* Create no data record                                  */
         %no_record_htm(Msg1 = %superq(No_scenario_changes_msg1),
            msg2= %superq(No_scenario_changes_msg2),
            msg3=, 
            htm_name=body);
            %let trans_rc = 999999;
         %return;
   %end;

    /* -------------------------------------------------------------------
      GET SCENARIO NAMES OF CHANGED SCENARIOS
      ------------------------------------------------------------------- */
    PROC SQL;
    CREATE TABLE work.distinct_names AS
        SELECT distinct(a.SCENARIO_NAME)
        FROM work.updated_scenarios AS a
        ORDER BY a.SCENARIO_NAME;
    QUIT;
    /* -------------------------------------------------------------------
      SELECT ORGINALS OF CHANGED SCENARIOS (version = 1 )
      ------------------------------------------------------------------- */
    PROC SQL;
    CREATE TABLE WORK.version_one AS
        SELECT a.*
        FROM seg_kc.FSK_SCENARIO AS a   
            INNER JOIN WORK.DISTINCT_NAMES AS b ON (a.SCENARIO_NAME = b.SCENARIO_NAME)
        WHERE a.VERSION_NUMBER = 1;
    QUIT;
    /* -------------------------------------------------------------------
      APPEND ORIGINAL TO CHANGED
      ------------------------------------------------------------------- */
    PROC SQL;
    CREATE TABLE WORK.all_updated_scenarios AS
        SELECT * FROM WORK.UPDATED_SCENARIOS
           OUTER UNION CORR
        SELECT * FROM WORK.VERSION_ONE;
    QUIT;
    /* -------------------------------------------------------------------
      SORT
      ------------------------------------------------------------------- */
    PROC SORT   DATA=WORK.ALL_UPDATED_SCENARIOS
          OUT=WORK.ALL_UPDATED_SCENARIOS_SORTED;
          BY SCENARIO_NAME VERSION_NUMBER;
    RUN;
    /* -------------------------------------------------------------------
      CREATE MACRO VARIABLE LISTS AND COUNTS
      ------------------------------------------------------------------- */
    %global scenario_list scenario_id_list version_number_list header_id_list scenario_count;
    PROC SQL noprint;
    SELECT a.SCENARIO_NAME,
            max(a.scenario_id) ,
            max(a.version_number) ,
            max(a.header_id)
            INTO :scenario_list separated by ',' ,
            :scenario_id_list separated by ',' ,
            :version_number_list separated by ','   ,
            :header_id_list separated by ','
        FROM WORK.ALL_UPDATED_SCENARIOS_SORTED AS a
        GROUP BY a.SCENARIO_NAME
        ORDER BY a.SCENARIO_NAME;
    QUIT;

    PROC SQL noprint;
    SELECT count( distinct a.SCENARIO_NAME)
        INTO :scenario_count
        FROM WORK.ALL_UPDATED_SCENARIOS_SORTED AS a;
    QUIT;
   %mend aml_select_updated_scenarios;


   %macro aml_process_scenario_specs;

   /* -------------------------------------------------------------------
      Sort data set WORK.ALL_UPDATED_SCENARIOS_SORTED
      ------------------------------------------------------------------- */

    Data WORK.ALL_UPDATED_SCENARIOS_LABELED;
    set WORK.ALL_UPDATED_SCENARIOS_SORTED;
    KEEP
        SCENARIO_SHORT_DESCRIPTION
        SCENARIO_DESCRIPTION
        SCENARIO_CODE_LOCATION_DESC
        _SCENARIO_CATEGORY_CODE
        _PRODUCT_TYPE_CODE
        _SCENARIO_STATUS_CODE
        SCENARIO_RUN_FREQUENCY_DESC
        _CURRENT_IND
        INITIAL_ROUTE_ID
        ROUTE_TYPE_CODE
        DFLT_SUPPRESS_DURATION_COUNT
        SCENARIO_DURATION_COUNT
        MONEY_LAUNDERING_BAYES_WEIGHT
        TERROR_FINANCING_BAYES_WEIGHT
        EXECUTION_PROBABILITY_RATE
        _RISK_FACTOR_IND
        _LOGICAL_DELETE_IND
        ROUTING_GROUP_ID
        END_DATE
        END_USER_ID
        SCENARIO_NAME
        VERSION_NUMBER
        CREATE_DATE
        CREATE_USER_ID ;

/*--------------------------------------------------------------------
    Generate Formats
---------------------------------------------------------------------*/

        _SCENARIO_CATEGORY_CODE = put(SCENARIO_CATEGORY_CODE,$scencat.);
        _PRODUCT_TYPE_CODE      = put(PRODUCT_TYPE_CODE,$product.);
        _SCENARIO_STATUS_CODE   = put(SCENARIO_STATUS_CODE,$scnstat.);
        _CURRENT_IND            = put(CURRENT_IND,$yesno.);
        _RISK_FACTOR_IND        = put(RISK_FACTOR_IND,$yesno.);
        _LOGICAL_DELETE_IND     = put(LOGICAL_DELETE_IND,$yesno.);
    RUN;
    PROC SORT DATA=WORK.ALL_UPDATED_SCENARIOS_LABELED OUT=WORK.SORTED_SPECS ;
        BY SCENARIO_NAME VERSION_NUMBER CREATE_DATE CREATE_USER_ID;
    RUN;
   /* -------------------------------------------------------------------
      TRANSPOSE SCENARIOS SPECIFICATIONS
      ------------------------------------------------------------------- */

   PROC TRANSPOSE DATA=WORK.SORTED_SPECS
      OUT=WORK.TRANSPOSED_SPECS
      PREFIX=Column
      NAME=Source
      LABEL=Label
   ;
      BY SCENARIO_NAME VERSION_NUMBER CREATE_DATE CREATE_USER_ID;
      VAR
            SCENARIO_SHORT_DESCRIPTION
            SCENARIO_DESCRIPTION
            _SCENARIO_CATEGORY_CODE
            _PRODUCT_TYPE_CODE
            _SCENARIO_STATUS_CODE
            SCENARIO_RUN_FREQUENCY_DESC
            INITIAL_ROUTE_ID
            ROUTE_TYPE_CODE
            DFLT_SUPPRESS_DURATION_COUNT
            SCENARIO_DURATION_COUNT
            MONEY_LAUNDERING_BAYES_WEIGHT
            TERROR_FINANCING_BAYES_WEIGHT
            EXECUTION_PROBABILITY_RATE
            _RISK_FACTOR_IND
            ROUTING_GROUP_ID
            _CURRENT_IND
            _LOGICAL_DELETE_IND ;
   RUN;
   /* -------------------------------------------------------------------
      LOOP THROUGH ALL CHANGED FSK_SCENARIO_PARAMETER TABLES
      LOOKING FOR CHANGED PARAMETERS
      ---------------------------------------------------------------- */
     %do i = 1 %to &scenario_count ;
       %let sn = %scan(%quote(&scenario_list),&i);
       %let scenid = %scan(%quote(&scenario_id_list),&i);
           %do oldv = 1 %to %scan(%quote(&version_number_list),&i) - 1;
           %let newv = %eval(&oldv +1);
                   /* -------------------------------------------------------------
                      SELECT OLDER OF CHANGED FSK_SCENARIO
                      -------------------------------------------------------- */
                   PROC SQL;
                   CREATE TABLE WORK.specs_&scenid._V&OLDV AS
                   SELECT a.SCENARIO_NAME                         ,
                      a.VERSION_NUMBER as ORIGINAL_VERSION        ,
                      a.CREATE_DATE as ORIGINAL_CREATE_DATE       ,
                      a.CREATE_USER_ID as ORIGINAL_CREATE_USER    ,
                      a.Source AS COLUMN                          ,
                      a.Column1 as ORIGINAL_VALUE
                    FROM WORK.TRANSPOSED_SPECS AS a
                    WHERE ( a.SCENARIO_NAME = "&SN" AND a.VERSION_NUMBER =  &OLDV )
                   ORDER BY a.SCENARIO_NAME, a.SOURCE;
                   QUIT;
                   /* -----------------------------------------------------------
                      SELECT NEWER OF CHANGED FSK_SCENARIO
                      ------------------------------------------------------ */
                   PROC SQL;
                   CREATE TABLE WORK.specs_&scenid._V&NEWV AS
                   SELECT a.SCENARIO_NAME                         ,
                      a.VERSION_NUMBER as UPDATED_VERSION         ,
                      a.CREATE_DATE as UPDATED_CREATE_DATE        ,
                      a.CREATE_USER_ID as UPDATED_CREATE_USER     ,
                      a.Source AS COLUMN                          ,
                      a.Column1 as UPDATED_VALUE
                    FROM WORK.TRANSPOSED_SPECS AS a
                    WHERE ( a.SCENARIO_NAME = "&SN" AND a.VERSION_NUMBER =  &NEWV )
                   ORDER BY a.SCENARIO_NAME, a.SOURCE;
                   QUIT;
                   /* -------------------------------------------------------------------
                      CREATE MACRO VARIABLES FOR USER_ID AND DATE - NEEDED WHEN A
                      PARAMETER IS DELETED - NEW RECORD DOES NOT HAVE USER_ID AND DATE
                      ------------------------------------------------------------------- */
                   PROC SQL NOPRINT;
                   SELECT MAX(a.CREATE_DATE)          FORMAT=DATETIME20.0,
                           MAX(a.CREATE_USER_ID)
                           INTO :UPDATE_DATE,
                           :UPDATE_USER
                    FROM WORK.ALL_UPDATED_SCENARIOS_SORTED  AS a
                       WHERE SCENARIO_NAME = "&SN" AND VERSION_NUMBER = &NEWV;
                   QUIT;

                   /* -------------------------------------------------------------------
                      MERGED OLD AND NEWER CHANGED PARMS
                      ------------------------------------------------------------------- */
                   DATA WORK.MERGED_SPECS;
                   MERGE  WORK.specs_&scenid._V&OLDV  WORK.specs_&scenid._V&NEWV;
                   BY SCENARIO_NAME  COLUMN;

                   /* -------------------------------------------------------------------
                      FIND CHANGED PARMS
                      ------------------------------------------------------------------- */
                   DATA WORK.CHANGED_SPECS;
                   SET  WORK.MERGED_SPECS;
                       DROP COLUMN
                           ORIGINAL_VERSION
                           ORIGINAL_CREATE_DATE
                           ORIGINAL_CREATE_USER
                           updated_create_user
                           updated_create_DATE  ;
                   iF strip(original_value) ne strip(updated_value) then do;
                       PARAMETER = column;
                       ATTRIBUTE = ' ';
                       PARAMETER_TYPE = 'Property';
                       sort_key = 1;
                       if updated_version = .          then updated_version = &newv;
                       if updated_create_user = ' '    then updated_create_user = "&update_user";
                       if updated_create_DATE = .      then   updated_create_DATE = input("&UPDATE_DATE",DATETIME20.0);
                       USER = updated_create_user;
                       CHANGE_DATE = updated_create_DATE;
                       /* eliminate changes before date_limit */
                       IF datepart(change_date) lt &date_limit then delete;
                       output WORK.CHANGED_SPECS;
                   end;
                   run;
                   /* -------------------------------------------------------------------
                      APPEND TO SAVE THESE RECORD
                      ------------------------------------------------------------------- */
                   PROC APPEND BASE=WORK.CHANGED_RECORDS  DATA=WORK.CHANGED_SPECS FORCE
                        FORCE;
                   RUN;
       %end;
   %end;
   %mend aml_process_scenario_specs;


   %macro aml_process_scenario_parameters;
   /* -------------------------------------------------------------------
      SELECT PARAMETERS OF CHANGED SCENARIOS
      ------------------------------------------------------------------- */
   PROC SQL;
   CREATE TABLE work.select_parms AS
   SELECT a.SCENARIO_NAME     ,
       a.VERSION_NUMBER       ,
       a.CREATE_DATE          ,
       b.PARM_NAME            ,
       b.SCENARIO_ID          ,
       b.PARM_VALUE           ,
       b.PARM_TYPE_DESC       ,
       b.PARM_DESC            ,
       a.CREATE_USER_ID 
   FROM seg_kc.FSK_SCENARIO_PARAMETER AS b  
        INNER JOIN WORK.ALL_UPDATED_SCENARIOS_SORTED AS a ON (b.SCENARIO_ID = a.SCENARIO_ID)
    ORDER BY a.SCENARIO_NAME, a.VERSION_NUMBER, a.CREATE_DATE, b.PARM_NAME, b.SCENARIO_ID, b.PARM_VALUE;
   QUIT;
   /* -------------------------------------------------------------------
      Sort data set WORK.SELECT_PARMS
      ------------------------------------------------------------------- */
   PROC SORT
      DATA=WORK.SELECT_PARMS
       (KEEP=PARM_VALUE
           PARM_TYPE_DESC
           PARM_DESC
           SCENARIO_NAME
           VERSION_NUMBER
           SCENARIO_ID
           CREATE_DATE
           CREATE_USER_ID
           PARM_NAME)
      OUT=WORK.SELECT_PARMS_SORTED
      ;
      BY SCENARIO_NAME VERSION_NUMBER SCENARIO_ID CREATE_DATE CREATE_USER_ID PARM_NAME;
   RUN;
   /* -------------------------------------------------------------------
      TRANSPOSE SELECTED PARM ENTRIES BY PARM_NAME
      ------------------------------------------------------------------- */
   PROC TRANSPOSE DATA=WORK.SELECT_PARMS_SORTED
      OUT=WORK.TRANSPOSED_PARMS(LABEL="Transposed WORK.SELECT_PARMS")
      PREFIX=Column
      NAME=Source
      LABEL=Label
   ;
      BY SCENARIO_NAME VERSION_NUMBER SCENARIO_ID CREATE_DATE CREATE_USER_ID PARM_NAME;
      VAR PARM_VALUE PARM_TYPE_DESC PARM_DESC;
   /* -------------------------------------------------------------------
      LOOP THROUGH ALL CHANGED FSK_SCENARIO_PARAMETER TABLES
      LOOKING FOR CHANGED PARAMETERS
      ------------------------------------------------------------------- */
     %do i = 1 %to &scenario_count ;
       %let sn = %scan(%quote(&scenario_list),&i);
       %let scenid = %scan(%quote(&scenario_id_list),&i);
           %do oldv = 1 %to %scan(%quote(&version_number_list),&i) - 1;
           %let newv = %eval(&oldv +1);
                   /* -------------------------------------------------------------------
                      SELECT OLDER OF CHANGED FSK_SCENARIO_PARAMETER
                      ------------------------------------------------------------------- */
                   PROC SQL;
                   CREATE TABLE WORK.parms_&scenid._V&OLDV AS
                   SELECT a.SCENARIO_NAME                      ,
                      a.VERSION_NUMBER as ORIGINAL_VERSION     ,
                      a.CREATE_DATE as ORIGINAL_CREATE_DATE    ,
                      a.CREATE_USER_ID as ORIGINAL_CREATE_USER ,
                      a.PARM_NAME as SCENARIO_PARAMETER        ,
                      a.Source AS COLUMN                       ,
                      a.Column1 as ORIGINAL_VALUE
                    FROM WORK.TRANSPOSED_PARMS AS a
                    WHERE ( a.SCENARIO_NAME = "&SN" AND a.VERSION_NUMBER =  &OLDV)
                   ORDER BY a.SCENARIO_NAME, a.PARM_NAME, a.SOURCE;
                   QUIT;
                   /* -------------------------------------------------------------------
                      SELECT NEWER OF CHANGED FSK_SCENARIO_PARAMETER
                      ------------------------------------------------------------------- */
                   PROC SQL;
                   CREATE TABLE WORK.parms_&scenid._V&NEWV AS
                   SELECT a.SCENARIO_NAME                      ,
                      a.VERSION_NUMBER as UPDATED_VERSION      ,
                      a.CREATE_DATE as UPDATED_CREATE_DATE     ,
                      a.CREATE_USER_ID as UPDATED_CREATE_USER  ,
                      a.PARM_NAME as SCENARIO_PARAMETER        ,
                      a.Source AS COLUMN                       ,
                      a.Column1 as UPDATED_VALUE
                    FROM WORK.TRANSPOSED_PARMS AS a
                    WHERE ( a.SCENARIO_NAME = "&SN" AND a.VERSION_NUMBER =  &NEWV)
                   ORDER BY a.SCENARIO_NAME, a.PARM_NAME, a.SOURCE;
                   QUIT;

                   /* -------------------------------------------------------------------
                      CREATE MACRO VARIABLES FOR USER_ID AND DATE - NEEDED WHEN A
                      PARAMETER IS DELETED - NEW RECORD DOES NOT HAVE USER_ID AND DATE
                      ------------------------------------------------------------------- */
                   PROC SQL NOPRINT;
                   SELECT MAX(a.CREATE_DATE) ,
                           MAX(a.CREATE_USER_ID)
                           INTO :UPDATE_DATE,
                           :UPDATE_USER
                    FROM WORK.ALL_UPDATED_SCENARIOS_SORTED  AS a
                       WHERE SCENARIO_NAME = "&SN" AND VERSION_NUMBER = &NEWV;
                   QUIT;


                   /* -------------------------------------------------------------------
                      MERGED OLD AND NEWER CHANGED PARMS
                      ------------------------------------------------------------------- */
                   DATA WORK.MERGED_PARMS;
                   MERGE  WORK.parms_&scenid._V&OLDV  WORK.parms_&scenid._V&NEWV;
                   BY SCENARIO_NAME SCENARIO_PARAMETER COLUMN;
                   /* -------------------------------------------------------------------
                      FIND CHANGED PARMS
                      ------------------------------------------------------------------- */
                   DATA WORK.CHANGED_PARMS;
                   SET  WORK.MERGED_PARMS;
                       DROP scenario_parameter
                           column
                           original_version
                           original_create_date
                           original_create_user
                           updated_create_user
                           updated_create_date  ;
                        FORMAT parameter_type  $20.
                               attribute $35.;   
                   if strip(original_value) ne strip(updated_value) then do;

                       parameter =  upcase(scenario_parameter);
                       attribute = column;
                       parameter_type = 'Parameter';
                       sort_key = 2;
                       if updated_version = .          then updated_version = &newv;
                       if updated_create_user = ' '    then updated_create_user = "&update_user";
                       if updated_create_DATE = .      then   updated_create_DATE = input("&UPDATE_DATE",DATETIME20.0);
                       user = updated_create_user;
                       change_date = updated_create_date;
                       /* eliminate changes before date_limit */
                       if datepart(change_date) lt &date_limit then delete;
                       OUTPUT WORK.CHANGED_PARMS;
                   end;
                   run;
                   /* -----------------------------------------------------------
                      APPEND TO SAVE THESE RECORD
                      ------------------------------------------------------- */
                   PROC APPEND BASE=WORK.CHANGED_RECORDS  DATA=WORK.CHANGED_PARMS FORCE
                        FORCE;
                   RUN;
       %end;
   %end;
   %mend aml_process_scenario_parameters;


   %macro aml_print_scenario_audit_report;
   /* ---------------------------------------------------------*/
   /* Setup the output environment
   /* ---------------------------------------------------------*/
   /* Delete output directory if it exists
   /* use lower case 'deldir' to indicate that you want to
   /* remove directory - macro converts to proper code
   /* depending on platform  - unix or windows
   /* ---------------------------------------------------------*/
        /* If output directory exists --- remove old reports      */
        %if %sysfunc(fileexist(&outputDirectory)) %then %do;
            %aml_execute_command(deldir  &outputDirectory);
        %end;

        %aml_execute_command(mkdir &outputDirectory);
        %aml_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _amlmsg_=ERROR: AML: Unable to create directory: &outputDirectory;
            %put_aml_error;
            %RETURN;
        %end;


       /* -------------------------------------------------------------------
          CHECK FOR NUMBER OF RECORDS TO PRINT
          ------------------------------------------------------------------- */
        PROC SQL;
        CREATE TABLE work.changed_records_sorted AS
        SELECT a.SCENARIO_NAME ,
              a.PARAMETER_TYPE ,
              a.PARAMETER ,
              a.ATTRIBUTE ,
              a.ORIGINAL_VALUE ,
              a.UPDATED_VALUE ,
              a.USER ,
              a.CHANGE_DATE ,
              a.UPDATED_VERSION ,
              a.SORT_KEY 
        FROM WORK.CHANGED_RECORDS AS a
        WHERE a.SORT_KEY > 0
        ORDER BY a.SCENARIO_NAME, a.CHANGE_DATE, a.UPDATED_VERSION, a.SORT_KEY, a.PARAMETER;
        QUIT;

       %if &sqlobs = 0 %then %do;
       /* ------------------------------------------------------------------------------
           NO RECORDS TO PRINT --- Put warning message and put html page with message. 
       --------------------------------------------------------------------------------- */
           %let _amlmsg_=NOTE: AML: Zero records selected for this time period, no report will be produced;
           %put_aml_error;
           /* If output directory exists --- remove old reports      */
           %if %sysfunc(fileexist(&outputDirectory)) %then %do;
               %aml_execute_command(deldir  &outputDirectory);
           %end;
           /* Create new output directory                            */
           %aml_execute_command(mkdir &outputDirectory);
           %aml_rcset(&syserr);
           %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
               %let _amlmsg_=ERROR: AML: Unable to create directory: &outputDirectory;
               %put_aml_error;
           %end;
           /* Create no data record                                  */
           %no_record_htm(Msg1 = %superq(No_scenario_changes_msg1),
               msg2= %superq(No_scenario_changes_msg2),
               msg3=, 
               htm_name=body);

           %let trans_rc = 999999;
           %RETURN;
       %end;

       /* -------------------------------------------------------------------
          Setup for printing report.
          ------------------------------------------------------------------- */
        DATA WORK.CHANGED_RECORDS_SORTED_MOD;
        SET WORK.CHANGED_RECORDS_SORTED;



        SELECT (ATTRIBUTE);
            WHEN ('PARM_TYPE_DESC')     ATTRIBUTE = "&format";
            WHEN ('PARM_VALUE')         ATTRIBUTE = "&value";
            WHEN ('PARM_DESC')          ATTRIBUTE = "&desc";
            WHEN (' ')
                DO;
                    IF PARAMETER = 'SCENARIO_DESCRIPTION' THEN ATTRIBUTE = "&desc";
                    ELSE  ATTRIBUTE = "&value";
                END;
           OTHERWISE                   ATTRIBUTE = "&value";
        END;
        IF ORIGINAL_VALUE EQ ' ' and UPDATED_VALUE NE ' ' then Action = "&add";
        IF ORIGINAL_VALUE NE ' ' and UPDATED_VALUE EQ ' ' then Action = "&delete";
        IF ORIGINAL_VALUE NE ' ' and UPDATED_VALUE NE ' ' then Action = "&change";
        RUN;


        PROC SQL;
        CREATE TABLE WORK.SCENARIO_DESC AS
        SELECT  SCENARIO_NAME,
                SCENARIO_SHORT_DESCRIPTION
        FROM WORK.ALL_UPDATED_SCENARIOS_SORTED
        GROUP BY SCENARIO_NAME
        HAVING SCENARIO_ID = MAX(SCENARIO_ID)
        ORDER BY SCENARIO_NAME;
        QUIT;

        PROC SQL;
        CREATE TABLE WORK.PRINT_DATASET AS
            SELECT A.*,
            B.SCENARIO_SHORT_DESCRIPTION
        FROM CHANGED_RECORDS_SORTED_MOD AS A
        LEFT JOIN WORK.SCENARIO_DESC AS B
        ON (a.SCENARIO_NAME = B.SCENARIO_NAME)

        ORDER BY SCENARIO_NAME, SCENARIO_SHORT_DESCRIPTION, UPDATED_VERSION;
        QUIT;

        DATA WORK.PRINT_DATASET;
        SET WORK.PRINT_DATASET;
        %label_maker(basefile=scenario_audit_meta
               ,table_name=SCENARIO_AUDIT_RPT_FORMATS_META
               ,generate=FORMAT) ;
        SCENARIO_DESCRIPTION =CATX(' ',SCENARIO_NAME, ' - ',SCENARIO_SHORT_DESCRIPTION);


        PROC SORT DATA=WORK.PRINT_DATASET OUT=WORK.PRINT_DATASET_SORTED;
            BY SCENARIO_DESCRIPTION UPDATED_VERSION CHANGE_DATE PARAMETER;
        RUN;

   
        /* -------------------------------------------------------------------
            Set ODS and GRAPH parms
        ------------------------------------------------------------------- */
      OPTIONS   device=&outputdevice  ;
      GOPTIONS  xpixels=480  ypixels=360;

      ODS LISTING CLOSE;
      ODS HTML PATH="&outputDirectory"     (URL=NONE)
              BODY="&outputBody"
              NOGTITLE NOGFOOTNOTE
              STYLE=&_outputstyle._prt   ;

        /* -------------------------------------------------------------------
           Print Report
        ------------------------------------------------------------------- */

        OPTIONS NOBYLINE;

        PROC PRINT DATA=WORK.PRINT_DATASET_SORTED
            NOOBS UNIFORM
            LABEL    ;
        VAR CHANGE_DATE
            USER
            PARAMETER
            ORIGINAL_VALUE
            UPDATED_VALUE
            Action
            ATTRIBUTE
           ;
    
        BY SCENARIO_DESCRIPTION ;
        PAGEBY SCENARIO_DESCRIPTION;
        ID UPDATED_VERSION ;
    /*--------------------------------------------------------------------
        Generate LABELS
    ---------------------------------------------------------------------*/
    %label_maker(basefile=scenario_audit_meta
            ,table_name=SCENARIO_AUDIT_RPT_LABEL_META
            ,generate=LABEL) ;

        TITLE1 "&title";
        TITLE2 "#byval1";
        RUN;
        /* -------------------------------------------------------------------
          End of report.
          ------------------------------------------------------------------- */
       TITLE; FOOTNOTE;
       RUN;
OPTIONS BYLINE;

   ODS _ALL_ CLOSE;
   ODS LISTING;
   RUN;
   %mend aml_print_scenario_audit_report;



   /* ---------------------------------------------------------------------------- */
   /*   THESE SIX MACROS COMPRISE THE ENTIRE SCENARIO_CHANGE_AUDIT_REPORT
   /* ---------------------------------------------------------------------------- */
   %aml_init_and_verify_params;
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do ;
         %if &trans_rc = 999999 %then %let trans_rc = 0 ;
         %goto MACRO_END ;
      %end;
   %aml_select_updated_scenarios;
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do ;
         %if &trans_rc = 999999 %then %let trans_rc = 0 ;
         %goto MACRO_END ;
      %end;
   %aml_process_scenario_specs;
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do ;
         %if &trans_rc = 999999 %then %let trans_rc = 0 ;
         %goto MACRO_END ;
      %end;
   %aml_process_scenario_parameters;
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do ;
         %if &trans_rc = 999999 %then %let trans_rc = 0 ;
         %goto MACRO_END ;
      %end;
   %aml_print_scenario_audit_report;
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do ;
         %if &trans_rc = 999999 %then %let trans_rc = 0 ;
         %goto MACRO_END ;
      %end;



   %MACRO_END:   /*  EXIT MACRO */

      /* Delete title macro variable */
      %label_maker(basefile=scenario_audit_meta
            ,table_name=REPORT_TITLE_MACRO_META
            ,generate=MACDEL) ;

      %if &trans_rc eq 3 or &trans_rc ge 5 %then
         %execute_macro(amlerror);
      %else %do;
         %execute_macro(post&_amlmodule_);
         %execute_macro(amlsuccess);
      %end;

%mend scenario_audit_report;
