/******************************************************************************/
/*                  S A S   S A M P L E   L I B R A R Y                       */
/*                                                                            */
/*    NAME: weekly_trend_report.sas                                           */
/*   TITLE: Weekly Trend Report                                               */
/* PRODUCT: SAS Anti-Money Laundering                                         */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/******************************************************************************/
/* Description: Produces a weekly bar chart of the number of alert items
/*              created each week.  Also a optional tabular report. Default
/*              is to produce tabular report. See operational parms below to
/*              turn it off.  Number of weeks to select and start day of week
/*              can be selected by parms.
/*
/* PARAMETERS:
/* <!-- ODS Parms -->
/*        outputDirectory = DIRECTORY FOR THIS REPORT
/*        outputBody = REPORT HTML BODY NAME:  body.htm
/*        outputDevice = GRAPHICS RENDERING:  javaimg/actximg
/*        _outputStyle = STYLE TO APPLY:  sasweb, banker etc
/* <!-- Report Parms -->
/*        num_weeks = NUMBER OF WEEKS TO TREND: 10
/*        start_day_of_week = STARTING DAY OF WEEK 1=SUN., 7=SAT. :  1
/*        title = REPORT TITLE
/*        bar_color = BAR_COLOR:  blue
/*        bar_shape = SHAPE OF BARS:  block
/*        table_report_YN>N = PRODUCE A AN APPENDED TABULATE REPORT:  Y
/*        display_x_axis_label_YN display x-axis label: Y
/*        display_Y_axis_label_YN display y-axis label: Y
/* <!-- Called routines and macros -->
/*      %aml_rcset
/*      %put_aml_error
/*      %aml_execute_command
/*      %label_maker
/*
/******************************************************************************/

/* ************************************************************************** */
/*  Copyright (c) 2012 by SAS Institute Inc., Cary, NC 27513 USA              */
/*  ---All Rights Reserved.                                                   */
/* ************************************************************************** */

%macro weekly_trend_report ;

   /* Set Module Name */
    %local _AMLMODULE_;
    %let _AMLMODULE_=WEEKLY_TREND_REPORT;

   /* Call preWEEKLY_TREND_REPORT stub macro */
    %execute_macro(pre&_amlmodule_);

   /* Set error code to 0 and message to blank*/
    %let trans_rc = 0;
    %let _amlmsg_=;

   /*--------------------------------------------------------
    * Make sure the AML start datetime has been set.
    * Assign its value to macro variable aml_start_datetime.
    *--------------------------------------------------------*/
    %aml_start_datetime_read;
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %goto MACRO_END2;
    %end;


/* -------------------------------------------------------------------
    Generate Macro Variables
--------------------------------------------------------------------- */
    /* Get title from xml file*/
    %label_maker(basefile=weekly_trend_meta
            ,table_name=REPORT_TITLE_MACRO_META
            ,generate=MACVAR) ;

    %label_maker(basefile=weekly_trend_meta
            ,table_name=ALERTS_BY_WEEK_META
            ,generate=MACVAR) ;

   /* Verrify and set all report parameters  */
   DATA _NULL_;

   %let _start_day_of_week=1;                           /* OPTIONAL MACRO VARIABLE - set default */
   %IF %SYMEXIST(start_day_of_week) = 1 %then %do ;    /* Does variable exist?    */
       %IF &start_day_of_week ne   %then %do ;         /* Does it have a value?   */
           %let _start_day_of_week=&start_day_of_week;               /* Then set value          */
       %end;
   %end;

   %let _num_weeks=10;                          /* OPTIONAL MACRO VARIABLE - set default */
   %IF %SYMEXIST(num_weeks) = 1 %then %do ;    /* Does variable exist?    */
       %IF &num_weeks ne   %then %do ;         /* Does it have a value?   */
           %let _num_weeks=&num_weeks;               /* Then set value          */
       %end;
   %end;

   %let _table_report_YN = Y;                        /* OPTIONAL MACRO VARIABLE - set default */
   %IF %SYMEXIST(table_report_YN) = 1  %then %do ;  /* Does variable exist?    */
       %IF &table_report_YN ne %then %do ;          /* Does it have a value?   */
           %if ( %upcase(&table_report_YN) = Y or
           %upcase(&table_report_YN) = N )  %then %do ;
               %let _table_report_YN = &table_report_YN;     /* Then set value          */
           %end;
       %end;
   %end;

   %let _bar_shape = block;                    /* OPTIONAL MACRO VARIABLE - set default */
   %IF %SYMEXIST(bar_shape) = 1  %then %do ;  /* Does variable exist?    */
       %IF &bar_shape ne %then %do ;          /* Does it have a value?   */
           %let _bar_shape =  &bar_shape;      /* Then set value          */
       %end;
   %end;
   %let _bar_color = blue;                    /* OPTIONAL MACRO VARIABLE - set default */
   %IF %SYMEXIST(bar_color) = 1  %then %do ;  /* Does variable exist?    */
       %IF &bar_color ne %then %do ;          /* Does it have a value?   */
           %let _bar_color =  &bar_color;       /* Then set value          */
       %end;
   %end;

   %let _display_x_axis_label_YN = Y ;                    /* OPTIONAL MACRO VARIABLE - set default */
   %if %symexist(display_x_axis_label_YN) = 1 %then %do ;  /* Does variable exist?    */
      %if &display_x_axis_label_YN ne %then %do ;          /* Does it have a value?   */
         %if ( %upcase(&display_x_axis_label_YN) = Y
         or %upcase(&display_x_axis_label_YN) = N )  %then %do ;
            %let _display_x_axis_label_YN = &display_x_axis_label_YN ;     /* Then set value          */
         %end ;
      %end ;
   %end ;

   %let _display_y_axis_label_YN = Y ;                    /* OPTIONAL MACRO VARIABLE - set default */
   %if %symexist(display_y_axis_label_YN) = 1  %then %do ;  /* Does variable exist?    */
      %if &display_y_axis_label_YN ne %then %do ;          /* Does it have a value?   */
         %if ( %upcase(&display_y_axis_label_YN) = Y
         or %upcase(&display_y_axis_label_YN) = N )  %then %do ;
            %let _display_y_axis_label_YN = %upcase(&display_y_axis_label_YN) ;     /* Then set value          */
         %end ;
      %end ;
   %end ;

   DATA _NULL_;
   /* ---------------------------------------------------------*/
   /* SET MACRO VARIABLES FOR WEEK RANGES AND TITLES
   /* ---------------------------------------------------------*/
   todays_date = datepart(&aml_start_datetime);

   /* Get first day of range            */
   call symput('Begin_range',put(intnx("week.%trim(%left(&_start_day_of_week))"
   ,todays_date,-&_num_weeks+1,'B'),8.));
   /* Get very last day of range        */
   call symput('End_range',put(intnx("week.%trim(%left(&_start_day_of_week))"
   ,todays_date,-0,'E'),8.));

   /* Get First day of beginning  week  */
   call symput('Begin_week',put(intnx("week.%trim(%left(&_start_day_of_week))"
   ,todays_date,-&_num_weeks+1,'B'),&format_nldate));
   /* Get First day of last week        */
   call symput('End_week',put(intnx("week.%trim(%left(&_start_day_of_week))"
   ,todays_date,-0,'B'),&format_nldate));
   run;

   /* ---------------------------------------------------------*/
   /* CREATE A TEMPLATE FILE OF ALL WEEKS IN RANGE
   /* ---------------------------------------------------------*/
   data work.ZERO_TEMPLATE;
   keep week_date ;
   format week_date &format_nldate;
   do i = 0 to &_num_weeks - 1;
       week_date = intnx("week.&_start_day_of_week",&begin_range,i);
      * zero_value = 0;
       output zero_template;
   end;
   run;
   %aml_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _amlmsg_= NOTE: AML: Unable to create output table: ZERO_TEMPLATE;
      %put_aml_error;
      %goto MACRO_END;
   %end;

/* ---------------------------------------------------------*/
/* ASSIGN CREATE_DATES TO WEEK BUCKETS AND SUMMARIZE
/* ---------------------------------------------------------*/
   %if %sysfunc(exist(seg_kc.fsk_alert)) %then %do;
      data work.WEEK_BUCKETS;
      set seg_kc.fsk_alert;
         keep week_date count;
         format  week_date &format_nldate;
         week_date = intnx("week.&_start_day_of_week",datepart(create_date),0,'B');
         count=1;
         if  &Begin_range <= week_date <= &End_range  then output  work.week_buckets;
      run;
      %aml_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _amlmsg_= NOTE: AML: Unable to create output table: WEEK_BUCKETS table;
         %put_aml_error;
         %goto MACRO_END;
      %end;
   %end;
   %else %do;
      %let _amlmsg_= NOTE: AML: seg_kc.FSK_ALERT does not exist or is not accessible;
      %put_aml_error;
      %goto MACRO_END;
   %end;
   %let dsid=%sysfunc(open(work.WEEK_BUCKETS,i));
    %let num_obs=%sysfunc(attrn(&dsid,nobs));
    %let rc=%sysfunc(close(&dsid));
   %if &num_obs = 0 %then %do;
      %let _amlmsg_= NOTE: AML: Zero records selected for this report, no report will be produced;
      %put_aml_error;
      %goto MACRO_END;
   %end;

   proc means  data=work.week_buckets
       noprint
       chartype
       maxdec=0
       nway
       vardef=df
       sum nonobs  ;
       var count;
       class week_date /   order=unformatted ascending;
      output  out=work.summarized sum()= /   ways inherit   ;
   run;
   %aml_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _amlmsg_= NOTE: AML: Unable to create output table: SUMMARIZED;
      %put_aml_error;
      %goto MACRO_END;
   %end;

/* ---------------------------------------------------------*/
/* FILL IN MISSING WEEKS WITH ZEROS FROM TEMPLATE
/* ---------------------------------------------------------*/
   data work.chart_data;
   merge  work.summarized   work.zero_template   ;
       by week_date;
      drop _way_ _type_ _freq_;
   if count = . then count= 0;
   run;
   %aml_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _amlmsg_= NOTE: AML: Unable to create output table: CHART_DATA;
      %put_aml_error;
      %goto MACRO_END;
   %end;

   PROC MEANS  DATA=WORK.CHART_DATA NOPRINT CHARTYPE MAXDEC=1
      VARDEF=DF MEAN MIN MAX SUM NONOBS  ;
   VAR count;
   OUTPUT  OUT=work.total_avg_min_max
           MEAN()=average
           MIN()=minimum
           MAX()=maximum
           SUM()=total
       /  INHERIT  ;
   run;
   %aml_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _amlmsg_= NOTE: AML: Unable to create output table: TOTAL_AVG_MIN_MAX;
      %put_aml_error;
      %goto MACRO_END;
   %end;



   data _null_;
   set work.total_avg_min_max;
      call symput('average',put(average,8.1));
      call symput('minimum',put(minimum,8.));
      call symput('maximum',put(maximum,8.));
      call symput('total',put(total,8.));
   run;
   %aml_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _amlmsg_= NOTE: AML: Unable to access table: TOTAL_AVG_MIN_MAX;
      %put_aml_error;
      %goto MACRO_END;
   %end;


/* -------------------------------------------------------------------
    Generate Title Macro Variables
---------------------------------------------------------------------- */
    %label_maker(basefile=weekly_trend_meta
                ,table_name=ALERTS_BY_WEEK_TITLE_META
                ,generate=macvar) ;


/* ---------------------------------------------------------*/
/* Setup the output environment
/* ---------------------------------------------------------*/
/* Delete output directory if it exists
/* use lower case 'deldir' to indicate that you want to
/* remove directory - macro converts to proper code
/* depending on platform  - unix or windows
/* ---------------------------------------------------------*/
    %if %sysfunc(fileexist(&outputDirectory)) %then %do;
        %aml_execute_command(deldir  &outputDirectory);
    %end;
   /* Create output directory                                  */
   %aml_execute_command(mkdir &outputDirectory);
   %aml_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _amlmsg_= NOTE: AML: Unable to create directory: &outputDirectory;
      %put_aml_error;
      %goto MACRO_END;
   %end;
   OPTIONS   device=&outputdevice  ;
   GOPTIONS  xpixels=480  ypixels=360;

   /* Set ODS parms                                            */
   ODS LISTING CLOSE;
   ODS HTML PATH="&outputDirectory"     (URL=NONE)
           BODY="&outputBody"
           NOGTITLE NOGFOOTNOTE
           STYLE=&_outputstyle   ;

   /* ---------------------------------------------------------*/
   /* BAR Chart
   /* ---------------------------------------------------------*/
   TITLE;
   FOOTNOTE;
   %if &_display_x_axis_label_YN eq N  %then %do ;
      %let xaxis_label = ;
   %end;
   %if &_display_y_axis_label_YN eq N  %then %do ;
      %let yaxis_label = ;
   %end;

   Axis1 COLOR=BLACK STYLE=1 WIDTH=1 LABEL=(HEIGHT=12pt JUSTIFY=Right "&yaxis_label");
   Axis2 COLOR=BLACK STYLE=1 WIDTH=1 LABEL=(HEIGHT=12pt JUSTIFY=LEFT "&xaxis_label");
   PATTERN1 COLOR=&_bar_color;
   TITLE1 "&title";
   TITLE2 "&title2";
   FOOTNOTE1   height=&_fheight  "&footnote1";
   FOOTNOTE2   height=&_fheight  "&footnote2";


   /*  FOOTNOTE8 height=&_fheight justify=left "&aml_footnote_date" justify=right "&aml_footnote_version";*/
   PROC GCHART DATA=WORK.CHART_DATA;


   VBAR3D   week_date
      / SUMVAR=Count
         SHAPE=&_bar_shape
         DISCRETE
         TYPE=SUM
         NOLEGEND
      RAXIS=AXIS1
      MAXIS=AXIS2 LREF=34 CREF=BLACK AUTOREF   ;
format week_date  &format_nldate;
   RUN;
   %aml_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _amlmsg_= NOTE: AML: Unable to access table: CHART_DATA;
      %put_aml_error;
      %goto MACRO_END;
   %end;
   TITLE; FOOTNOTE;

/* ---------------------------------------------------------*/
/* Output table of Values
/* ---------------------------------------------------------*/
/*Do we want a table report  - default is Y  */
%if %upcase(&_table_report_YN) = Y %then %do;
   PROC TABULATE DATA=WORK.CHART_DATA
      STYLE={FONT_SIZE=2 FONT_WEIGHT=MEDIUM FONT_STYLE=ROMAN JUST=CENTER VJUST=MIDDLE} ;
      VAR count;
      CLASS week_date / MISSING ORDER=UNFORMATTED;
      TABLE /* Row Dimension */
         week_date={LABEL=''}
         ALL={LABEL="&total_alerts"},
         /* Column Dimension */
         count={LABEL="&number_of_alerts" STYLE={NOBREAKSPACE=ON}}*(
             Sum={LABEL="&alerts" STYLE={NOBREAKSPACE=ON}}*F=&format_nlnum9
         PctSum={LABEL="&percent_of_total" STYLE={NOBREAKSPACE=ON}}*F=&format_nlnum51)
         /* Table Options */
         / BOX={LABEL="&week_beginning" STYLE={JUST=CENTER VJUST=MIDDLE NOBREAKSPACE=ON}};
   TITLE;
   FOOTNOTE;
   TITLE1 "&title";
   TITLE2 "&title2";
   FOOTNOTE1   height=&_fheight  "&footnote1";
   FOOTNOTE2   height=&_fheight  "&footnote2"    ;

   /* FOOTNOTE8 height=&_fheight justify=left "&aml_footnote_date" justify=right "&aml_footnote_version";*/
   RUN;
   %aml_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _amlmsg_= NOTE: AML: Unable to access table: CHART_DATA;
      %put_aml_error;
      %goto MACRO_END;
   %end;
   TITLE;FOOTNOTE;
%end;

   /* End of task code. */
   ODS _ALL_ CLOSE;
   ODS LISTING;
   %MACRO_END:   /*  EXIT MACRO */
      /* Delete report macro variables so you don't get macro variable collisions   */
      %symdel title
         start_day_of_week   _start_day_of_week
         bar_shape           _bar_shape
         bar_color           _bar_color
         num_weeks           _num_weeks
         table_report_YN     _table_report_YN    / nowarn ;
      /*--------------------------------------------------------------------
          SYMDEL Macro Variables
      ---------------------------------------------------------------------*/
          %label_maker(basefile=weekly_trend_meta
                  ,table_name=ALERTS_BY_WEEK_META
                  ,generate=MACDEL) ;
      /*--------------------------------------------------------------------
          SYMDEL Title Macro Variables
      ---------------------------------------------------------------------*/
          %label_maker(basefile=weekly_trend_meta
                  ,table_name=ALERTS_BY_WEEK_TITLE_META
                  ,generate=MACDEL) ;

        /* Delete title macro variable */
         %label_maker(basefile=weekly_trend_meta
                ,table_name=REPORT_TITLE_MACRO_META
                ,generate=MACDEL) ;

      %if &trans_rc eq 3 or &trans_rc ge 5 %then
         %execute_macro(amlerror);
      %else %do;
         %execute_macro(post&_amlmodule_);
         %execute_macro(amlsuccess);
      %end;

%mend weekly_trend_report;
